local addonName, XUB = ...;

local done=nil

function XUB_ApplyProfile(myProfileName)

	if ( myProfileName == "" ) then
		XUB_ShowMessage("XUB: Error: No previous profile was found.  You have to use '/XUB p <profile>' with a profile name at least once so that mod can remember what it is." );
		return;
	end

	if ( InCombatLockdown() ) then
		XUB.ChangesToBeApplyAfterCombat = myProfileName;
		XUB_ShowMessage("XUB: Changes can not be applied while in Combat, but they will be applied after combat");
		return;
	end

	XUB.Busy = 1;

	XUB_RemoveButtons();
	
	-- 	collectgarbage("collect");
	
	local myCurrentProfileInfo = XUB.ProfileDescription[myProfileName]
	
	if myCurrentProfileInfo then
		XUB_Initialize_Membertables(myCurrentProfileInfo);

		for i=1,#XUB.ConfiguredUnits do
			XUB_ApplyProfile_for_SingleUnit(myProfileName, XUB.ConfiguredUnits[i])
		end
	else
		XUB_ShowMessage("XUB: Warning: Profile " .. CCC(myProfileName).." is not configured.")
	end

	-- define how long all buttons are displayed after applying a profile
	-- or in other words: define how long there will be no OnUpdate-cycles after applying a profile
	if XUB.manualChange then
		XUB.NextUpdate = XUB.delay_after_manualProfileChange;
	else
		XUB.NextUpdate = XUB.delay_after_automaticProfileChange;
	end
	

	if ( XUB.FloatConfigured ) then
		getglobal("XUB_FloatingFrame"):Show();
	else
		getglobal("XUB_FloatingFrame"):Hide();
	end

	XUB.CurrentProfile = myProfileName;
	if XUB.ChangesToBeApplyAfterCombat then
		-- not announcement
	else
		XUB_ShowMessage("XUB.CurrentProfile: "..XUB.CurrentProfile);
	end
	XUB.ChangesToBeApplyAfterCombat = nil;


	XUB.Busy = 0;

-- 	collectgarbage("collect");

end

function XUB_ApplyProfile_for_SingleUnit(myProfileName, myUnit)

	if ( myProfileName == "" ) then
		XUB_ShowMessage("XUB: Error: No previous profile was found.  You have to use '/XUB p <profile>' with a profile name at least once so that XUB can remember what it is." );
		return;
	end

	-- Check if we have frames for myUnit
	local myFrameList, myWidth, myAlwaysGenerate, myCustomFrameUnit
	local myFrames = {};
	myFrameList, myWidth, myAlwaysGenerate, myCustomFrameUnit = XUB_Get_UnitProperties(myUnit)
	if (myFrameList == nil) or (myFrameList == "") then
		XUB_ShowDebugMessage("XUB: Warning: no UnitFrame found for " .. CCC(myUnit));
		return;
	end
	myFrames = { strsplit("/", myFrameList) };

	local myCR = 0;
	local myCurrentXOffset = 0
	local myCurrentYOffset, myCurrentGroupname
	local myCurrentProfileInfo = XUB.ProfileDescription[myProfileName]
	local l_myCurrentAttachUnit

	local myShortUnit = myUnit
	-- if the current unit is 'raid1' or 'party5', then shorten it to 'raid' or 'party'
	if string.find(myUnit, '%d+') then
		myShortUnit = myUnit:sub(1,string.find(myUnit, '%d+')-1 )
	end

	local l_myLastXOffset = 0
	local l_myCROffset = 0
	-- now let's iterate through all lines of the current profile, each line consisting of a group
	for i=1,#myCurrentProfileInfo do
		if myCurrentXOffset ~= 0 then
			l_myLastXOffset = myCurrentXOffset
		end
		l_myCurrentAttachUnit = myCurrentProfileInfo[i].attachUnit;

		if (( l_myCurrentAttachUnit == "raid" ) and ( XUB.RaidUnitframesConfigured == nil )) then
			-- do nothing, we will not apply groups in that case
		elseif (l_myCurrentAttachUnit == myShortUnit ) or ((l_myCurrentAttachUnit == "party") and ((myUnit == "player_in_party") or (myUnit == "playerpet_in_party"))) then
			myCurrentGroupname = myCurrentProfileInfo[i].groupName;
			myCurrentXOffset = l_myCROffset + myCurrentProfileInfo[i].xOffset;
			myCurrentYOffset = myCurrentProfileInfo[i].yOffset;
			
			if ( myCurrentGroupname == "CR" ) then
				myCR = 1;
				l_myCROffset = l_myLastXOffset
			else
				XUB_CreateCoordsForButtons(myUnit,myCR,myCurrentXOffset,myCurrentYOffset,myWidth)
				myCR =0;					-- reset the CR
				l_myCROffset = 0

				XUB_ApplyGroup(myUnit, myCurrentGroupname, myFrames, myWidth, myAlwaysGenerate, myCustomFrameUnit);
			end
		end
	end
end

function XUB_ApplyGroup(myUnit, myCurrentGroupname, myFrames, myWidth, myAlwaysGenerate, myCustomFrameUnit)

	local myDestinationUnit, myExcludeUnits

	local myFrameFound = nil;
	local myButtonCreated = nil;

	local myDestinationUnit_is_in_XUB_UPDATEUNITS = nil;

	local myCurrentGroup = XUB.GroupDescription[myCurrentGroupname]
	if myCurrentGroup == nil then
		XUB_ShowMessage("XUB Error: Could not find a GROUP definition for group: '" .. myCurrentGroupname .."'");
		return;
	end
	
	-- now let's iterate through all lines of the current group, each line consisting of a button name
	for i = 1,#myCurrentGroup do

		myCurrentButtonInfo = XUB_GetButtonInfo(myCurrentGroup[i].buttonName)

		if myCurrentButtonInfo then

			myDestinationUnit = XUB_DecodeDestinationUnit(myCurrentGroup[i].destination, myCustomFrameUnit, myUnit)

			myExcludeUnits    = myCurrentGroup[i].excludeUnit
			
			myFrameFound = nil;
				-- now let's iterate through all the frames for the current unit
			for myFrameCounter=1,#myFrames do 
				if getglobal(myFrames[myFrameCounter]) then	-- if the frame exists
					myFrameFound = 1;
					if ( getglobal(myFrames[myFrameCounter]):IsVisible() ) or ( myAlwaysGenerate == 1) then

						-- Enter the destination unit of the current button to the update list
						if not XUB.Buttons_for_Dest[myDestinationUnit] then 	-- if there are no buttons for the current destiantion unit yet
							table.insert( XUB.UPDATEUNITS , myDestinationUnit )
						end

						XUB_MakeButton(myFrames[myFrameCounter],myUnit,myDestinationUnit,myExcludeUnits,myWidth,myCurrentButtonInfo);

--						myDestinationUnit_is_in_XUB_UPDATEUNITS = nil;
--						for counter=1,#XUB.UPDATEUNITS do
--							if ( XUB.UPDATEUNITS[counter] == myDestinationUnit ) then
--								myDestinationUnit_is_in_XUB_UPDATEUNITS = 1;
--								break;
--							end
--						end
--						if ( myDestinationUnit_is_in_XUB_UPDATEUNITS == nil ) then
--							table.insert( XUB.UPDATEUNITS , myDestinationUnit );
--						end;

					else
					end
				else	-- the current frame did not exist
					XUB_ShowDebugMessage("XUB: Unable to find " .. CCC(myFrames[myFrameCounter]) .. ". No buttons could be attached to that frame." );
				end
			end
			if ( not myFrameFound ) then	-- none of the frames for the current unit did exist
				XUB_ShowDebugMessage("XUB: Warning: no valid frame found for unit .. " .. CCC(myUnit) .. ". Frames: " .. CCC(myFrame) );
			end
			-- advance with the coordinates for buttons
			XUB.Coords_for_Next_Button[myUnit].x = XUB.Coords_for_Next_Button[myUnit].x + XUB.ButtonSize + XUB.BUTTON_SPACING
		else
			--XUB_ShowDebugMessage("XUB: No valid button part 2." );
		end	
	end
end


function XUB_MakeButton(myCurrentFrame,myUnit,myDestinationUnit,myExcludeUnits,myWidth,myCurrentButtonInfo)

	local myID = myCurrentButtonInfo.globalID
	local myButtonType = myCurrentButtonInfo.buttonType
	
	-- Increment the number used to uniquely identify the buttons (so we can do a getglobal later to find the frame/button).
	if XUB.Buttons_for_Dest[myDestinationUnit] then
		XUB.Buttons_for_Dest[myDestinationUnit] = XUB.Buttons_for_Dest[myDestinationUnit] + 1
	else
		XUB.Buttons_for_Dest[myDestinationUnit] = 1;
	end

	local myButtonName = XUB_GenerateButtonName(myDestinationUnit,XUB.Buttons_for_Dest[myDestinationUnit]);

	--XUB_ShowMessage( "Working with button " .. CCC(myButtonName));
	local myButton = getglobal(myButtonName);
	if ( myButton == nil ) then
		--XUB_ShowMessage("newly created button: " .. CCC(myButtonName));
--		myButton = CreateFrame("Button", myButtonName, UIParent,  "XUB_ActionButtonTemplate, SecureHandlerStateTemplate");
		myButton = CreateFrame("Button", myButtonName, UIParent,  "XUB_ActionButtonTemplate");

		local new_cooldown = getglobal(myButton:GetName()..'Cooldown');
		myButton.CooldownFrame = new_cooldown;

		XUB_Initialize_CountdownLayer_and_Font(myButton);
	else
		--XUB_ShowMessage("reused button: myButtonName: '" .. CCC(myButtonName) .. "'");
	end

	myButton:SetFrameStrata(XUB.FRAME_STRATA); --MEDIUM
	myButton:SetButtonState(XUB.BUTTON_STRATA); --NORMAL
	myButton:SetNormalTexture(myCurrentButtonInfo.texture)
	myButton:SetHighlightTexture(myCurrentButtonInfo.texture)
	myButton:SetWidth(XUB.ButtonSize);
	myButton:SetHeight(XUB.ButtonSize);
	myButton:SetScale(tonumber(myWidth) / XUB.ButtonSize);  

	local myUseButtonArray = {}
	table.insert( myUseButtonArray , "LeftButtonUp" );
	if XUB.Use_RightButton then
		table.insert( myUseButtonArray , "RightButtonUp" );
		XUB.Use_RightButton = "RightButtonUp"
	end
	if XUB.Use_MiddleButton then
		table.insert( myUseButtonArray , "MiddleButtonUp" );
		XUB.Use_MiddleButton = "MiddleButtonUp"
	end
	if XUB.Use_Button4 then
		table.insert( myUseButtonArray , "Button4Up" );
		XUB.Use_Button4 = "Button4Up"
	end
	if XUB.Use_Button5 then
		table.insert( myUseButtonArray , "Button5Up" );
		XUB.Use_Button5 = "Button5Up"
	end
	myButton:RegisterForClicks(myUseButtonArray[1], myUseButtonArray[2], myUseButtonArray[3], myUseButtonArray[4], myUseButtonArray[5]);

	--XUB.ButtonQueue[#XUB.ButtonQueue+1] = myButton;
	table.insert(XUB.ButtonQueue,myButton);

	------------------------------
	-- initialise Button values --
	------------------------------
	
	-- Static values (they do not change in the OnUpdate-Cycle)
	myButton.Multitarget         = ""
	myButton.FullSpellName       = ""
	myButton.SpecialLastSeconds  = nil;
	myButton.Blink               = 0;
	myButton.isMacro             = nil;
	myButton.isUse               = nil;
	myButton.isCastType          = nil;
	myButton.isSpell             = nil;
	myButton.isHeal              = nil;
	myButton.Threshold           = nil;
	myButton.isDebuff            = nil;
	myButton.isBuff              = nil;
	myButton.global_ID           = myID;					-- GlobalID of the button (SpellIF for Spells, ItemID for Items)
	myButton.Spell               = aSpellName;              -- Name of the spell on the button
	myButton.Bar                 = myUnit;                   -- The bar the button is on
	myButton.DestinationUnit     = myDestinationUnit;   	-- Name of the unit receiving the spell
	myButton.ExcludeUnits        = myExcludeUnits;           -- Units where the current button should be hidden
	myButton.AttachedFrame       = myCurrentFrame;                  -- Attached Frame
	

	-- Dynamic values (they may change in the OnUpdate-Cycle)
	myButton.is_STATE_ENABLED    = nil;
	myButton.is_STATE_HEALTHOFF  = nil;
	myButton.is_STATE_BUFFED     = nil;
	myButton.is_STATE_DIMMED     = nil;
	myButton.is_STATE_COOLDOWN   = nil;
	myButton.is_STATE_NO_ITEMS   = nil;
	myButton.is_STATE_OUTOFMANA  = nil;
	myButton.is_STATE_OUTOFRANGE = nil;
	myButton.is_STATE_INVALID    = nil;
	myButton.MyCooldown          = 0;
	myButton.LastsTill           = nil;
	myButton.ShowCountdown       = nil;
	myButton.State               = 0;
	
	XUB_CreateDetailedButtonInfo(myCurrentButtonInfo, myButton)
	
	if XUB.MB_TARGET then         --different mouseclicks cast to different targets
		myButton:SetAttribute("*unit2", "player")
		myButton:SetAttribute("*unit3", "target")
		myButton:SetAttribute("*unit4", "focus")
		myButton:SetAttribute("*unit*", myDestinationUnit)
		myButton.Multitarget = "MB-TARGET"
	elseif XUB.KB_TARGET then     --different keyboard modifier cast to different targets
		myButton:SetAttribute("alt-unit*", "player")
		myButton:SetAttribute("shift-unit*", "target")
		myButton:SetAttribute("ctrl-unit*", "focus")
		myButton:SetAttribute("*unit*", myDestinationUnit)
		myButton.Multitarget = "KB-TARGET"
	else
		myButton:SetAttribute("*unit2", myDestinationUnit)
		myButton:SetAttribute("*unit3", myDestinationUnit)
		myButton:SetAttribute("*unit4", myDestinationUnit)
		myButton:SetAttribute("alt-unit*", myDestinationUnit)
		myButton:SetAttribute("shift-unit*", myDestinationUnit)
		myButton:SetAttribute("ctrl-unit*", myDestinationUnit)
		myButton:SetAttribute("*unit*", myDestinationUnit)
		myButton.Multitarget = ""
	end

	local myAnchor = XUB.ANCHOR
	if ( myUnit == "float" ) then
		myAnchor = "TOPRIGHT"
	end	
	myButton:SetPoint("TOPLEFT", myCurrentFrame, myAnchor , XUB.Coords_for_Next_Button[myUnit].x , XUB.Coords_for_Next_Button[myUnit].y );

	myButton.isActive = 1;                       -- For removed buttons it would be 0
	myButton:Enable();
	myButton:Show();
	myButton:SetAlpha(1);

	if 1==1 then
		return
	end
	
	RegisterUnitWatch(myButton);
	myButton:SetAttribute("_onstate-XUBstate", [[           -- arguments: self, stateid, newstate
	if newstate == "hostile" then
		print("now hostile")
		if string.find( string.lower(myButton.ExcludeUnits .. "/") , "hostile/" ) then -- and hostile units are excluded
			self.is_STATE_INVALID = 1;
			self:Hide();
		end
	elseif newstate == "friendly" then
		print("now friendly")
		if string.find( string.lower(myButton.ExcludeUnits .. "/") , "friendly/" ) then -- and friendly units are excluded
			self.is_STATE_INVALID = 1;
			self:Hide();
		end
	elseif newstate == "notarget" then
		print("now hidden")
		self:Hide();
	elseif newstate == "yestarget" then
		print("now shown")
	end
	]]);
	XUB_ShowMessage("been here with "..CCC(myButton.FullSpellName))
	RegisterStateDriver(myButton, "XUBstate", "[target=exists] yestarget, [target=noexists] notarget, [target=help] friendly, [target=harm] hostile");
end
