local addonName, XUB = ...;

function XUB_CreateCoordsForButtons(myUnit,myCR,myXOffset,myYOffset,myWidth)
	if ( XUB.Coords_for_Next_Button[myUnit] == nil ) then            -- if unit has no buttons yet then
		XUB.Coords_for_Next_Button[myUnit] = { x = (myXOffset * XUB.ButtonSize) / myWidth , y = (myYOffset* XUB.ButtonSize) / myWidth };
	else
		XUB.Coords_for_Next_Button[myUnit].x = myXOffset + XUB.Coords_for_Next_Button[myUnit].x + XUB.GROUPSPACING;
		XUB.Coords_for_Next_Button[myUnit].y = myYOffset + XUB.Coords_for_Next_Button[myUnit].y;
	end

	if ( myCR == 1 ) then
		XUB.Coords_for_Next_Button[myUnit].x = (myXOffset * XUB.ButtonSize) / myWidth
		XUB.Coords_for_Next_Button[myUnit].y = XUB.Coords_for_Next_Button[myUnit].y - XUB.ButtonSize - XUB.BUTTON_SPACING;
	end
end

function XUB_DecodeDestinationUnit(myDestinationUnit, myCustomFrameUnit, myUnit)
	if ( myDestinationUnit == "*" ) then
		myDestinationUnit = myCustomFrameUnit or myUnit;
	end
	if ( myDestinationUnit == "float" ) or ( myDestinationUnit == "party" ) or ( myDestinationUnit == "partypet" ) or ( myDestinationUnit == "raid" ) or ( myDestinationUnit == "arena" )then
		XUB_ShowMessage("XUB: Error: The target of a button can not be 'float', 'party', 'partypet', 'raid' or 'arena'");
		return
	end
	if myDestinationUnit == "mb-target" then
		myDestinationUnit = myUnit
		XUB.MB_TARGET = 1;
	elseif myDestinationUnit == "kb-target" then
		myDestinationUnit = myUnit
		XUB.KB_TARGET = 1;
	else
		XUB.KB_TARGET = nil;
		XUB.MB_TARGET = nil;
	end
	if myDestinationUnit == "float" then 		-- because the multi target buttons are place on the float frame
		myDestinationUnit = "target"
	end

	return myDestinationUnit
end

function XUB_GetButtonInfo(myButtonName)

	local myCurrentButtonInfo
	local myDummyTable = XUB.ButtonDescription[myButtonName]
	
	if myDummyTable == nil then
		XUB_ShowDebugMessage("XUB: Warning: No button found with the name "..CCC(myButtonName)..". Dual spec ?")
		return nil
	end
	
	myCurrentButtonInfo = myDummyTable[1]
	
	if ( myCurrentButtonInfo.globalID == 0 ) and ( myCurrentButtonInfo.buttonType ~= "macro" ) then
		-- then we have a spell/item, that is not valid at the moment. Stop here
		XUB_ShowDebugMessage("XUB: Warning: Spell/item "..CCC(myButtonName).." is not vaild atm.")
		return nil
	end
	
	return myCurrentButtonInfo;
end

function XUB_CreateDetailedButtonInfo(myCurrentButtonInfo, myButton)

--[[
internals of all Buttons:

Buttons[1]: SpellID / ItemID (or "MacroID" for macros)
Buttons[2]: SpellTexture / ItemTexture / MacroIcon
Buttons[3]: button name
Buttons[4]: buttonType (spell, spellblink, buff, buffblink, debuff, debuffblink, health, healthblink, always, use or macro)
Buttons[5]: spellname / useItem / macroIconNumber 
Buttons[6]: condition (health: threshold | macro: macrotext | buff/debuff/useItem: condition)

All non-custom buttons have buttonType 'spell'
--]]

local myToolTip,mySpecialLastSeconds
local myButtonType = myCurrentButtonInfo.buttonType

	if ( myButtonType == "macro" ) then
		myButton.isMacro = 1;
		myButton:SetAttribute('type','macro');

		local myMacrotext = myCurrentButtonInfo.condition

		if string.find(string.lower(myMacrotext),"unit#") then
			myMacrotext = string.gsub(myMacrotext,"Unit#","unit#"); 	-- Unit lowercased
			myMacrotext = string.gsub(myMacrotext,"unit#",myButton.DestinationUnit);
		else    
			myButton.DestinationUnit = "player";       --macro has destination unit 'player', so that the macro buttons stay visible all the time
		end    

		-- !!!  string.gsub "\n" does NOT work !!!!!!
		-- So I used  'string.char(92)..string.char(110)' instead
		-- to make it easier writing lines we also replace spaces (byte[32]) here
		myMacrotext = string.gsub(myMacrotext,string.char(32)..string.char(92)..string.char(110)..string.char(32),string.char(10));
		myMacrotext = string.gsub(myMacrotext,string.char(32)..string.char(92)..string.char(110),string.char(10));
		myMacrotext = string.gsub(myMacrotext,string.char(92)..string.char(110)..string.char(32),string.char(10));

		myButton:SetAttribute('*macrotext*', myMacrotext );

		myButton.ToolTip = "Macro: \n\n" .. myMacrotext;

	elseif ( myButtonType == "use" ) then
		myButton.isUse = 1;
		myButton:SetAttribute('type','item');
		myButton:SetAttribute('*item*', myCurrentButtonInfo.action );

		myButton.UseItemName = myCurrentButtonInfo.action;

		if ( myCurrentButtonInfo.condition ) then 		-- if there is a condition for the button
			if ( string.find(myCurrentButtonInfo.condition,"!") ) then
				myButton.ToolTip = "Use item '" .. myCurrentButtonInfo.action .. "' on " .. myButton.DestinationUnit .. "\n\nDo only show if debuffed with '" .. string.sub(myCurrentButtonInfo.condition,2) .. "'";
			else
				myButton.ToolTip = "Use item '" .. myCurrentButtonInfo.action .. "' on " .. myButton.DestinationUnit .. "\n\nDo not show if buffed with '" .. myCurrentButtonInfo.condition .. "'";
			end
		else
			myButton.ToolTip = "Use item '" .. myCurrentButtonInfo.action .. "' on " .. myButton.DestinationUnit;
		end
		myButton.Condition = XUB_SetConditions(myCurrentButtonInfo.condition)
	else		-- some kind of spell
		myButton.isCastType = 1;
		myButton.FullSpellName = myCurrentButtonInfo.action;
		myButton:SetAttribute("type", "spell")
		myButton:SetAttribute("*spell*", myButton.FullSpellName )
		myButton.Condition = XUB_SetConditions(myCurrentButtonInfo.condition)

		if ( myButtonType == "spell" ) then
			myButton.ExtraType = "SPELL";
			myButton.isSpell = 1;
			myButton.ToolTip = "Shows as buffed if the spell is already on the unit"
		elseif ( myButtonType == "spellblink" ) then
			myButton.ExtraType = "SPELLBLINK";
			myButton.isSpell = 1;
			myButton.Blink = 1;
			myButton.ToolTip = "\nStarts blinking if the button is not a buff or debuff on the specified unit.";
		elseif ( myButtonType == "always" ) then
			myButton.ExtraType = "ALWAYS";
			myButton.isAlways = 1;
			myButton.ToolTip = "\nShow always, do not hide.";
		elseif ( myButtonType == "health" ) then
			myButton.ExtraType = "HEALTH";
			myButton.isHeal = 1;
			myButton.Threshold = tonumber(myCurrentButtonInfo.condition);
			myButton.ToolTip = "\nVisible when Unit Damage >= " .. myCurrentButtonInfo.condition;
		elseif ( myButtonType == "healthblink" ) then
			myButton.isHeal = 1;
			myButton.Blink = 1;
			myButton.ExtraType = "HEALTHBLINK";
			myButton.Threshold = tonumber(myCurrentButtonInfo.condition);
			myButton.ToolTip = "\nBlinks when Unit Damage >= " .. myCurrentButtonInfo.condition;
		elseif ( string.find(myButtonType,"debuffblink" ) ) then
			myButton.ExtraType = "DEBUFFBLINK";
			myButton.isDebuff = 1;
			myButton.Blink = 1;
			myToolTip = "\nStarts blinking when unit has these debuffs: " .. myCurrentButtonInfo.condition;
			if string.len(myButtonType) > string.len("DEBUFFBLINK") then
				-- get the attached SpecialLastSeconds
				mySpecialLastSeconds = string.sub(myButtonType,string.len("DEBUFFBLINK")+1);
				if tonumber(mySpecialLastSeconds) then
					myToolTip = myToolTip .. "\nHandles the last " .. string.format("%d",mySpecialLastSeconds) .. "seconds of the buff differently.";
					myButton.SpecialLastSeconds = tonumber(mySpecialLastSeconds)
				end
			end
			myButton.ToolTip = myToolTip
		elseif ( string.find(myButtonType,"debuff" ) ) then
			myButton.ExtraType = "DEBUFF";
			myButton.isDebuff = 1;
			myToolTip = "\nShown when unit has these debuffs: " .. myCurrentButtonInfo.condition;
			if string.len(myButtonType) > string.len("DEBUFF") then
				-- get the attached SpecialLastSeconds
				mySpecialLastSeconds = string.sub(myButtonType,string.len("DEBUFF")+1);
				if tonumber(mySpecialLastSeconds) then
					myToolTip = myToolTip .. "\nHandles the last " .. string.format("%d",mySpecialLastSeconds) .. "seconds of the buff differently.";
					myButton.SpecialLastSeconds = tonumber(mySpecialLastSeconds)
				end
			end
			myButton.ToolTip = myToolTip
		elseif ( string.find(myButtonType,"buffblink" ) ) then
			myButton.ExtraType = "BUFFBLINK";
			myButton.isBuff = 1;
			myButton.Blink = 1;
			myToolTip = "\nStarts blinking if not buffed and not affected with: " .. myCurrentButtonInfo.condition;
			if string.len(myButtonType) > string.len("BUFFBLINK") then
				-- get the attached SpecialLastSeconds
				mySpecialLastSeconds = string.sub(myButtonType,string.len("BUFFBLINK")+1);
				if tonumber(mySpecialLastSeconds) then
					myToolTip = myToolTip .. "\nHandles the last " .. string.format("%d",mySpecialLastSeconds) .. "seconds of the buff differently.";
					myButton.SpecialLastSeconds = tonumber(mySpecialLastSeconds)
				end
			end
			myButton.ToolTip = myToolTip
		elseif ( string.find(myButtonType,"buff" ) ) then
			myButton.ExtraType = "BUFF";
			myButton.isBuff = 1;
			if myCurrentButtonInfo.condition:sub(1,1) ~= "$" then
				myToolTip = "\nNot shown if unit is also buffed or affected with: " .. myCurrentButtonInfo.condition;
			else
				myToolTip = "\nNot shown if unit is also buffed or affected with my own: " .. myCurrentButtonInfo.condition:sub(2);
			end
			if string.len(myButtonType) > string.len("BUFF") then
				-- get the attached SpecialLastSeconds
				mySpecialLastSeconds = string.sub(myButtonType,string.len("BUFF")+1);
				if tonumber(mySpecialLastSeconds) then
					myToolTip = myToolTip .. "\nHandles the last " .. string.format("%d",mySpecialLastSeconds) .. "seconds of the buff differently.";
					myButton.SpecialLastSeconds = tonumber(mySpecialLastSeconds)
				end
			end
			myButton.ToolTip = myToolTip
		end
	end

end

function XUB_Get_UnitProperties(aUnit)

	local myFrame = nil;
	local myWidth = 20;
	local myAlwaysGenerate = 0;
	local myCustomFrameUnit = nil;

-- CompactPartyFrameMember
-- CompactRaidGroup@Member#

	if ( aUnit == "player" ) then
		myFrame = XUB.UF["Player"].frame;
		myWidth = XUB.UF["Player"].width;
		myAlwaysGenerate = 1;
	elseif ( aUnit == "playerpet" ) then
		myFrame = XUB.UF["PlayerPet"].frame;
		myWidth = XUB.UF["PlayerPet"].width;
		myAlwaysGenerate = 1;
	elseif ( aUnit == "target" ) then
		myFrame = XUB.UF["Target"].frame;
		myWidth = XUB.UF["Target"].width;
		myAlwaysGenerate = 1;
	elseif ( aUnit == "focus" ) then
		myFrame = XUB.UF["Focus"].frame;
		myWidth = XUB.UF["Focus"].width;
		myAlwaysGenerate = 1;
	elseif ( aUnit == "float" ) then
		myFrame = "XUB_FloatingFrame";
		myWidth = XUB.UF_WIDTH_Float;
		myAlwaysGenerate = 1;
	elseif ( aUnit == "player_in_party" ) then
		myWidth = XUB.UF["Party"].width;
		myFrame = XUB.PartyFrameTable[aUnit];
		myCustomFrameUnit = "player"
	elseif ( aUnit == "playerpet_in_party" ) then
		myWidth = XUB.UF["PartyPet"].width;
		myFrame = XUB.PartyFrameTable[aUnit];
		myCustomFrameUnit = "pet"
	elseif ( string.sub(aUnit,1,8) == "partypet" ) then
		myWidth = XUB.UF["PartyPet"].width;
		myFrame = XUB.PartyFrameTable[aUnit];
	elseif ( string.sub(aUnit,1,5) == "party" ) then
		myWidth = XUB.UF["Party"].width;
		myFrame = XUB.PartyFrameTable[aUnit];
	elseif ( string.sub(aUnit,1,7) == "raidpet" ) then
		myFrame = ""
		myWidth = XUB.RF["RaidPet"].width;
	elseif ( string.sub(aUnit,1,4) == "raid" ) then
		myWidth = XUB.RF["RaidMember"].width;
		myFrame = XUB.RaidFrameTable[aUnit];
	elseif ( string.sub(aUnit,1,5) == "arena" ) then
		myWidth = XUB.RF["Arena"].width;
		myFrame = XUB.ArenaFrameTable[aUnit];
	elseif ( aUnit == XUB.UF["Custom1"].name ) then
		myFrame = XUB.UF["Custom1"].frame;
		myWidth = XUB.UF["Custom1"].width;
		testFrame = getglobal(myFrame);
		if ( testFrame ) then
			myCustomFrameUnit = testFrame:GetAttribute("unit")
		end
	elseif ( aUnit == XUB.UF["Custom2"].name ) then
		myFrame = XUB.UF["Custom2"].frame;
		myWidth = XUB.UF["Custom2"].width;
		testFrame = getglobal(myFrame);
		if ( testFrame ) then
			myCustomFrameUnit = testFrame:GetAttribute("unit")
		end
	elseif ( aUnit == XUB.UF["Custom3"].name ) then
		myFrame = XUB.UF["Custom3"].frame;
		myWidth = XUB.UF["Custom3"].width;
		testFrame = getglobal(myFrame);
		if ( testFrame ) then
			myCustomFrameUnit = testFrame:GetAttribute("unit")
		end
	elseif ( aUnit == XUB.UF["Custom4"].name ) then
		myFrame = XUB.UF["Custom4"].frame;
		myWidth = XUB.UF["Custom4"].width;
		testFrame = getglobal(myFrame);
		if ( testFrame ) then
			myCustomFrameUnit = testFrame:GetAttribute("unit")
		end
	elseif ( aUnit == XUB.UF["Custom5"].name ) then
		myFrame = XUB.UF["Custom5"].frame;
		myWidth = XUB.UF["Custom5"].width;
		testFrame = getglobal(myFrame);
		if ( testFrame ) then
			myCustomFrameUnit = testFrame:GetAttribute("unit")
		end
	else
		XUB_ShowMessage("XUB: Unknown unit '" .. CCC(aUnit) .. "'." );
	end

-- 	XUB_ShowMessage("XUB-debug: myFrame " .. CCC(myFrame));
-- 	XUB_ShowMessage("XUB-debug: myWidth " .. CCC(myWidth));
-- 	XUB_ShowMessage("XUB-debug: myAlwaysGenerate " .. CCC(myAlwaysGenerate));
-- 	XUB_ShowMessage("XUB-debug: myCustomFrameUnit " .. CCC(myCustomFrameUnit));
	return myFrame, myWidth, myAlwaysGenerate, myCustomFrameUnit;

end

function XUB_Initialize_CountdownLayer_and_Font(myButton)

	local myParent = myButton.CooldownFrame:GetParent();
	local myFontSize = 64;
	local myScale = math.min(myParent:GetWidth() / XUB.ButtonSize, 1);
	local myFont = STANDARD_TEXT_FONT;
	local myText;

	myButton.CooldownFrame.noCooldownCount = 1;

	myText = myButton.CooldownFrame:CreateFontString(nil, "OVERLAY");
	myText:SetPoint("CENTER", myButton.CooldownFrame, "CENTER", 0, 1);
	myText:SetFont(myFont, myFontSize * myScale * 1.2, "OUTLINE");
	myText:SetTextColor(1, 1, 1, 1);      -- white

	myButton.CooldownFrame.text = myText;
	myButton.CooldownFrame.text:SetText("");

end

function XUB_GenerateButtonName(aUnit,aIndex)
	return "XUB_" .. aUnit .. "_" .. aIndex;
end


function XUB_SetConditions(myTextCondition)

	if not myTextCondition then return; end

	local myConditionTable = {}
	local myConditionName, myConditionSelf, myConditionInvert
	
	local mycondition = { strsplit("/",string.lower(myTextCondition) ) };
	
	for i = 1,#mycondition do
		myConditionName = mycondition[i]
		if ( string.sub(myConditionName,1,1) == "$" ) then
			myConditionName = string.sub(myConditionName,2)
			myConditionSelf = 1
		else
			myConditionSelf = 0
		end
		if ( string.sub(myConditionName,1,1) == "!" ) then
			myConditionName = string.sub(myConditionName,2)
			myConditionInvert = true
		else
			myConditionInvert = nil
		end
		if ( string.sub(myConditionName,1,1) == "$" ) then 		-- not only $!, but also !$. Ugly hack, but works ;-))
			myConditionName = string.sub(myConditionName,2)
			myConditionSelf = 1
		else
			myConditionSelf = 0
		end
		table.insert(myConditionTable, {["name"] = myConditionName, ["selfCast"] = myConditionSelf, ["inverted"] = myConditionInvert} )
	end
	
	return myConditionTable
end