﻿local addonName, XUB = ...;

--[[
internals of all Buttons:

Buttons[1]: SpellID / ItemID (or 0 for macros)
Buttons[2]: SpellTexture / ItemTexture / MacroIcon
Buttons[3]: name of the button
Buttons[4]: buttonType (spell, spellblink, buff, buffblink, debuff, debuffblink, health, healthblink, use or macro)
Buttons[5]: spellname / macroIconNumber / useItem
Buttons[6]: buff: condition / debuff: condition / health: threshold / macro: macrotext / useItem: condition

All non-custom buttons have buttonType 'spell'.

Existing buttontypes are: 
Items: 'use'
Macros: 'macro'
Spells: 'spell', 'spellblink', 'buff', 'buffblink', 'debuff', 'debuffblink', 'health', 'healthblink', 'always'

--]]

local myCompleteCustomButtonInfo = {}
local foundCurrentProfile = nil;
local myInternalStandardSpellTable = {};
local TempProfile_for_Spec = XUB.Profile_for_Spec
local myCustomButtons_at_runtime = {}

--local allowedAttachFrames = { "player", "playerpet", "party", "partypet", "arena", "target", "focus", "raid", "raidpet" }


function XUB_Config_Open()
	XUB_GenerateDefaultSpellList();
	XUB_Fill_ConfigEditBoxes_with_Text()
	XUB_Config:Show();
end

function XUB_MakeGroups_OnClick()

	-- Get all custom buttons that are valid. 
	-- We'll need them to draw them in the button list in XUBSelectSpell.
	-- Do this silently without warnings if some of the buttons are not valid.
	XUB.SilentTest = 1;
	XUB_TestButtons();
	XUB.SilentTest = nil;

	XUB_Config:Hide();
	XUB_SelectButtonMaker:Show();
end

function XUB_Config_AddNewContentToGroups(myNewContentForGroups)

	local mySeparator = ""
	if  XUB_ConfigPanel2GroupsEdit:GetText() ~= "" then
		mySeparator = "\n"
	end

	if myNewContentForGroups ~= "" then
		XUB_ConfigPanel2GroupsEdit:SetText( XUB_ConfigPanel2GroupsEdit:GetText() .. mySeparator .. myNewContentForGroups );
	end
end

function XUB_Config_AddNewContentToProfiles(myNewContentForProfiles)

	local mySeparator = ""
	if  XUB_ConfigPanel3ProfilesEdit:GetText() ~= "" then
		mySeparator = "\n"
	end
	
	if myNewContentForProfiles ~= "" then
		XUB_ConfigPanel3ProfilesEdit:SetText( XUB_ConfigPanel3ProfilesEdit:GetText() .. mySeparator .. myNewContentForProfiles );
	end
end

function XUB_VerifyInputs()

	if ( XUB_TestButtons() == 0 ) and ( XUB_TestGroups() == 0 ) and ( XUB_TestProfiles() == 0 ) then
		XUB_ConfigSaveAndExit:Show();
		XUB_ConfigGetMacro:Show();
	else
		XUB_ConfigSaveAndExit:Hide();
		XUB_ConfigGetMacro:Hide();
	end

end

--
-- Here we generate a set of button definitions based on the spell book.
--
function XUB_GenerateDefaultSpellList()

	local SpellName, subSpellName, myNewButtonName
	local i=1
	local l_isPassive, l_Texture
	local myCurrentSpell = {}

	local l_SkillType, l_SpellId = GetSpellBookItemInfo(i,SpellBookFrame.bookType );
	l_SpellId = l_SpellId or 0;

	XUB.StandardButtons = {};
	myInternalStandardSpellTable = {}

	while ( l_SkillType ~= nil ) do

		l_isPassive = IsPassiveSpell(i, SpellBookFrame.bookType);
		if (l_SkillType == "SPELL") and (not l_isPassive) and (l_SpellId ~= 0) then
			SpellName, subSpellName = GetSpellInfo(i,SpellBookFrame.bookType );

			if SpellName and (SpellName ~= "") then
				-- let's create a name for the new button
				myNewButtonName = "SB_" .. SpellName;						-- 'SB' means: StandardButton
				myNewButtonName = string.gsub(myNewButtonName, " ", "");
				myNewButtonName = string.gsub(myNewButtonName, ":","" );
				myNewButtonName = string.gsub(myNewButtonName, "'","" );
				myNewButtonName = string.gsub(myNewButtonName, "%)","" );
				myNewButtonName = string.gsub(myNewButtonName, "%(","" );
		
				_, _, l_Texture = GetSpellInfo( l_SpellId )
				if ( l_Texture == nil ) or ( l_Texture == "" ) then
					l_Texture = "Interface\\Icons\\INV_Misc_QuestionMark";
				end
				
				myCurrentSpell = { l_SpellId, l_Texture, myNewButtonName, "spell", SpellName }
				

				--table.insert(XUB.StandardButtons, myCurrentSpell);
				myInternalStandardSpellTable[myNewButtonName] = myCurrentSpell;
			end
		end  

		i=i+1
		l_SkillType, l_SpellId = GetSpellBookItemInfo(i,SpellBookFrame.bookType );
		l_SpellId = l_SpellId or 0;
	end

	XUB.StandardButtons	= XUB_TableSort(myInternalStandardSpellTable);

end

function XUB_Fill_ConfigEditBoxes_with_Text()

	-- convert all the three arrays ( custom buttons, groups and profiles )
	-- to three large string containing all lines, and write the strings to the editboxes

	local myLoadText = "";
	local mySeperator = "";
	local myLastGroup = "";
	local myButtonType = "";

	if ( XUB.CustomButtons ~= nil ) then
		for i = 1,#XUB.CustomButtons do
			mySeperator = "";   						-- no separator ',' at the beginning of a line
			for e = 3,#XUB.CustomButtons[i] do
				myCurrentText = XUB.CustomButtons[i][e];
				if e == 5 then		-- if we are about to write the spellname/itemname
					-- all buttons except macro buttons have valid global IDs (either SpellID or ItemID)
					-- Instead of taking SpellName/ItemName from the config file, we are replacing
					-- it with the spellname/ItemName we get via ID.
					-- This allows us to share config files accross different languages
					myButtonType = XUB.CustomButtons[i][4];
					if (myButtonType == "macro") then
						myCurrentText = XUB.CustomButtons[i][e];
					elseif (myButtonType == "use") then
						myCurrentText = GetItemInfo(XUB.CustomButtons[i][1])  	-- take the name we get via ItemID
					else
						myCurrentText = GetSpellInfo(XUB.CustomButtons[i][1])  	-- take the name we get via SpellID
					end
					myCurrentText = myCurrentText or XUB.CustomButtons[i][e];
				end
				myLoadText = myLoadText .. mySeperator .. myCurrentText;
				mySeperator = ","; 						-- separator set to ','
			end
			myLoadText = myLoadText .. "\n" -- add a CarriageReturn to the string to separate the lines
		end

		XUB_ConfigPanel1ButtonsEdit:SetText( myLoadText );
		XUB_ConfigPanel1ButtonsEdit:SetCursorPosition(0);
	end


	--
	-- Groups
	--
	if ( XUB.ConfigGroups ~= nil ) then
		myLoadText = "";
		myLastGroup = "";
		local myExludeUnits = "";
		for i = 1,#XUB.ConfigGroups do					-- iterate through all groups
			mySeperator = "";														-- no separator ',' at the beginning of a line
			if ( XUB.ConfigGroups[i][1] ~= myLastGroup ) then		-- if we have a new groupname
				if ( myLastGroup ~= "" ) then											-- and if the new group is not the first group
					myLoadText = myLoadText .. "\n";								-- then add a CarriageReturn to seperate the groups
				end
				myLastGroup = XUB.ConfigGroups[i][1];
			end
	
			for e = 1,#XUB.ConfigGroups[i] do			-- iterate through all elements of a group line
--				myNewText = XUB.ConfigGroups[i][e]
--				if myNewText and (myNewText == "") and (e==4) then 	-- don't show an empty excludeUnit
--					myNewText = nil
--				end
--				if ( myNewText ~= nil ) then
--					myLoadText = myLoadText .. mySeperator .. myNewText;
--				end
				myLoadText = myLoadText .. mySeperator .. XUB.ConfigGroups[i][e];
				mySeperator = ","; 							-- separator set to ','
			end
			myLoadText = myLoadText .. "\n"		-- add a CarriageReturn to the string to separate the lines
		end

		XUB_ConfigPanel2GroupsEdit:SetText( myLoadText );
		XUB_ConfigPanel2GroupsEdit:SetCursorPosition(0);
	end

	--
	-- Profiles
	--
	if ( XUB.ConfigProfiles ~= nil ) then
		myLoadText = "";
		myLastGroup = "";
		for i = 1,#XUB.ConfigProfiles do					-- iterate through all profiles
			mySeperator = "";
			myMerge = XUB.ConfigProfiles[i][1] .. "~" .. XUB.ConfigProfiles[i][2];  -- create a combo of profilename and target frame for the separation check
			if ( myMerge ~= myLastGroup ) then
				if ( myLastGroup ~= "" ) then
					myLoadText = myLoadText .. "\n";
				end
				myLastGroup = myMerge;
			end
	
			for e = 1,#XUB.ConfigProfiles[i] do	-- iterate through all elements of the current line
--				myNewText = ""..XUB.ConfigProfiles[i][e]
--				if (e==4) or (e==5) then 	-- don't show xOffset/yOffset if it is 0
--					if XUB.ConfigProfiles[i][4]..XUB.ConfigProfiles[i][5] == "00" then
--						myNewText = nil
--					end
--				end
--				if ( myNewText ~= nil ) then
--					myLoadText = myLoadText .. mySeperator .. myNewText;
--				end
				myLoadText = myLoadText .. mySeperator .. XUB.ConfigProfiles[i][e];
				mySeperator = ","; 							-- separator set to ','
			end
			myLoadText = myLoadText .. "\n"
		end

		XUB_ConfigPanel3ProfilesEdit:SetText( myLoadText );
		XUB_ConfigPanel3ProfilesEdit:SetCursorPosition(0);
	end

end

--
-- Tests the syntax of the Button definition
--
function XUB_TestButtons()

	local myError = 0;
	local myButtonStructureError = nil;
	local myIsValidSpell
	local isSubTypeSpell = 1;
	local myArgumentsCount, myButtonType, myTexture, myID, myItems
	local myItemName, myItemLink
	myCompleteCustomButtonInfo = {}
	
	local myButtons = XUB_SplitTextIntoTable( XUB_ConfigPanel1ButtonsEdit:GetText() );
	
	for i = 1,#myButtons do
		myTrim,myItems = XUB_SplitCommaAndTrim(myButtons[i]);
		myButtonType = string.lower(myTrim[2]);
		myArgumentsCount = #myTrim;
		isSubTypeSpell = 1;			-- we reset this later for macro and use buttons
	
		if ( myButtonType == "macro" ) then
			isSubTypeSpell = nil;
			if ( myArgumentsCount < 4 ) then
				-- buttonName,macro,icon#,macro text
				XUB_ErrorMessage( "MACRO buttons require: buttonName,macro,icon#,macro text. "  .. CCC( myButtons[i] ) );
				myButtonStructureError = 1;
			else
				myID = 0;								-- set to 0 for all macors
				GetNumMacroIcons(); 		-- Required call
				myTexture = GetMacroIconInfo(myTrim[3]);
				if ( myTexture == nil ) or ( myTexture == "" ) then
					myTexture = "Interface\\Icons\\INV_Misc_QuestionMark";
				end
			end
		elseif ( myButtonType == "use" ) then
			isSubTypeSpell = nil;
			if ( myArgumentsCount ~= 3 ) and ( myArgumentsCount ~= 4 ) then
				-- buttonName,use,item name [,condition]
				XUB_ErrorMessage( "USE buttons require: buttonName,use,item name [,condition]. "  .. CCC( myButtons[i] ) );
				myButtonStructureError = 1;
			else
				myID, myTexture = XUB_UseItemCheck(myTrim[3],myTrim[1]);
			end	
		elseif ( myButtonType == "always" ) then
			if  ( myArgumentsCount ~= 3 ) then
					-- buttonName,always,name
					XUB_ErrorMessage( "ALWAYS buttons require: buttonName,always,name "  .. CCC( myButtons[i] ) );
					myButtonStructureError = 1;
			end
		elseif ( myButtonType == "spellblink" ) then
			if  ( myArgumentsCount ~= 3 ) then
				-- buttonName,spellblink,spellName
				XUB_ErrorMessage( "SPELLBLINK buttons require: buttonName,spellblink,spellName. "  .. CCC( myButtons[i] ) );
				myButtonStructureError = 1;
			end
		elseif ( myButtonType == "spell" ) then
			if  ( myArgumentsCount ~= 3 ) then
				-- buttonName,spell,spellName
				XUB_ErrorMessage( "SPELL buttons require: buttonName,spell,spellName. "  .. CCC( myButtons[i] ) );
				myButtonStructureError = 1;
			end
		elseif ( myButtonType == "healthblink" ) then
			if ( myArgumentsCount ~= 4 ) then
				-- buttonName,healthblink,spellName,threshold 
				XUB_ErrorMessage( "HEALTHBLINK buttons require: buttonName,healthblink,spellName,threshold. "  .. CCC( myButtons[i] ) );
				myButtonStructureError = 1;
			end
		elseif ( myButtonType == "health" ) then
			if ( myArgumentsCount ~= 4 ) then
				-- buttonName,health,spellName,threshold 
				XUB_ErrorMessage( "HEALTH  buttons require: buttonName,health,spellName,threshold. "  .. CCC( myButtons[i] ) );
				myButtonStructureError = 1;
			end
		elseif ( string.sub(myButtonType,1,11) == "debuffblink") then
			if ( myArgumentsCount ~= 4 ) then
				-- buttonName,debuffblink,spellName,condition
				UB_ErrorMessage( "DEBUFFBLINK buttons require: buttonName,debuffblink,spellName,condition. "  .. CCC( myButtons[i] ) );
				myButtonStructureError = 1;
			end
		elseif ( string.sub(myButtonType,1,6) == "debuff") then
			if ( myArgumentsCount ~= 4 ) then
				-- buttonName,debuff,spellName,condition
				XUB_ErrorMessage( "DEBUFF buttons require: buttonName,debuff,spellName,condition. "  .. CCC( myButtons[i] ) );
				myButtonStructureError = 1;
			end
		elseif ( string.sub(myButtonType,1,9) == "buffblink") then
			if ( myArgumentsCount ~= 4 ) then
				-- buttonName,buffblink,spellName,condition
				XUB_ErrorMessage( "BUFFBLINK buttons require: buttonName,buffblink,spellName,condition. "  .. CCC( myButtons[i] ) );
				myButtonStructureError = 1;
			end
		elseif ( string.sub(myButtonType,1,4) == "buff") then
			if ( myArgumentsCount ~= 4 ) then
				-- buttonName,buff,spellName,condition
				XUB_ErrorMessage( "BUFF buttons require: buttonName,buff,spellName,condition. "  .. CCC( myButtons[i] ) );
				myButtonStructureError = 1;
			end
		else
			XUB_ErrorMessage( "Unknown button specification - accepts: spell, spellblink, buff, buffblink, debuff, debuffblink, health, healthblink, always, use or macro." );
			myButtonStructureError = 1;
		end
	
		if myButtonStructureError then 	-- an error in the custom button structure can not work
			myError = 1;									-- No save allowed
			for i=1,myArgumentsCount do		-- List all arguments of the faulty line
				XUB_ErrorMessage("     " .. i .. CCC(myTrim[i]) );
			end
		else
			if ( isSubTypeSpell ) then 		-- if the current custom buttons is some kind of spell
											-- then lets have a look if it the spell is valid
				myIsValidSpell, myID, myTexture = XUB_CheckSpell(myTrim[3], myTrim[1])
				if (myIsValidSpell < 1) then	-- Seems something was wrong with the spell. Passive ?
					myError = 1;								-- No save allowed
				end
			end
		end
	
		if myInternalStandardSpellTable[myTrim[1]] then
			XUB_ErrorMessage("There is already an standard button with the name: "..myTrim[1])
			myError = 1;								-- No save allowed
		end
		if myCompleteCustomButtonInfo[myTrim[1]] then
			XUB_ErrorMessage("There is already a custom button with the name: "..myTrim[1])
			myError = 1;								-- No save allowed
		end
		
		-- while doing the check we got the ID and the Texture as byproducts.
		-- now let's store them in a temporary array so we do not have to refetch them at save
		myCompleteCustomButtonInfo[myTrim[1]] = { myID, myTexture, myTrim[1], myButtonType, myTrim[3], myTrim[4] }
	end
	
	return myError;
	
end

function XUB_TestGroups()

	-- XUB_ShowMessage("Testing Groups");		

	local myError = 0;
	local found = nil;
	local myTrim,myItems,currentCustomButtonName,myCustomButtons
	local myGroups = XUB_SplitTextIntoTable( XUB_ConfigPanel2GroupsEdit:GetText() );

	for i = 1,# myGroups do
		myTrim,myItems = XUB_SplitCommaAndTrim(myGroups[i]);
		if ( #myTrim ~= 3 ) and ( #myTrim ~= 4 ) then
			-- GroupName, buttonName, DestinationUnit [, ExcludeUnits]
			XUB_ShowMessage( "GROUP items require: GroupName, buttonName, DestinationUnit [, ExcludeUnits]. "  .. CCC( myGroups[i] ) );
			myError = 1;
		elseif  myTrim[1] == "CR" then
			XUB_ShowMessage("The group name 'CR is reserved. Please choose another name for your group.");
			myError = 1;
		elseif  myTrim[3] == "" then
			XUB_ShowMessage("The destination for a button cannot be empty." ..CCC( myGroups[i] ));
			myError = 1;
		else
			--search for the button that the current line refers to, by iterating through all buttons
			found = nil;
			-- first search through all standard buttons
			for ii = 1,#XUB.StandardButtons do
				if myTrim[2] == XUB.StandardButtons[ii][3] then
					found = 1;
					break;
				end
			end
			if not found then
				-- now search through all custom buttons
				myCustomButtons = XUB_SplitTextIntoTable( XUB_ConfigPanel1ButtonsEdit:GetText() );
				for ii = 1,#myCustomButtons do
					currentCustomButtonName = strsplit(",",myCustomButtons[ii])
					if myTrim[2] == currentCustomButtonName then
						found = 1;
						break;
					end
				end
			end
			if not found then
				XUB_ErrorMessage( "XUB: Warning: Group '" .. myTrim[1] .."' refers to the unknown button '" .. myTrim[2] .."'. A reason could be that you are using dualspec." );
			end
		end
	end

	return myError;

end

function XUB_TestProfiles()

	-- XUB_ShowMessage("Testing Profiles");		
	local myError = 0;
	local found = nil;
	local myTrim,myItems, currentGroupName, myGroups
	local myProfiles = XUB_SplitTextIntoTable( XUB_ConfigPanel3ProfilesEdit:GetText() );

	foundCurrentProfile = nil;

	for i = 1,#myProfiles do
		myTrim,myItems = XUB_SplitCommaAndTrim(myProfiles[i]);

		if ( #myTrim ~= 3 ) and ( #myTrim ~= 5 ) then
			-- ProfileName, attachFrame, GroupName [, X-Offset, Y-Offset]
			XUB_ShowMessage( "PROFILE items require: ProfileName, attachFrame, GroupName [, X-Offset, Y-Offset]. "  .. CCC( myProfiles[i] ) );
			myError = 1;
		elseif myTrim[3] == "CR" then
			-- then do nothing, because 'CR' is valid
		else
			--search for the group that the current line refers to, by iterating through all groups
			found = nil;
			myGroups = XUB_SplitTextIntoTable( XUB_ConfigPanel2GroupsEdit:GetText() );
			for ii = 1,#myGroups do
				currentGroupName = strsplit(",",myGroups[ii])
				if myTrim[3] == currentGroupName then
					found = 1;
					break;
				end
			end
			if not found then
				XUB_ShowMessage("Profile '" .. myTrim[1] .."' refers to the unknown group '" .. myTrim[3] .."'");
				myError = 1;
			end
--[[
			--search for attach frame that the current line refers to
			local l_wrong = 1;
			for i=1,#allowedAttachFrames do
				if myTrim[2]  == allowedAttachFrames[i] then
					l_wrong = nil;
					break
				end	
			end
			if l_wrong then
				XUB_ShowMessage("Profile '" .. myTrim[1] .."' refers to the unknown attach frame '" .. myTrim[2] .."'");
			end
--]]
			end
		if myTrim[1] == XUB.CurrentProfile then
			--XUB_ShowDebugMessage("current profile "..CCC(XUB.CurrentProfile).." still exists")
			foundCurrentProfile = 1;
		end
		-- check if the attach unit is valid
		
	end

	return myError;

end

function XUB_InitialiseProfileNames()

	XUB.profileNames = {}
	local myTempProfileNames = {}
	for i = 1,#XUB.ConfigProfiles do
		myTempProfileNames[XUB.ConfigProfiles[i][1]] = 1
	end
	for profileName in pairs(myTempProfileNames) do
		--XUB_ShowMessage("profileName: "..CCC(profileName))
		table.insert(XUB.profileNames,profileName)
	end
	table.sort(XUB.profileNames)

	if #XUB.profileNames > 1 and GetNumTalentGroups() > 1 then
		XUB_ConfigDefaultProfilesButton:Show();
	else
		XUB_ConfigDefaultProfilesButton:Hide();
		XUB.ProfileNames = {}
	end

	UIDropDownMenu_Initialize(XUB.LDB_Menu, function() XUB_LDB_Menu(); end)
	
end

function XUB_Config_Save()

	XUB_SaveButtons();
	XUB_SaveGroups();
	XUB_SaveProfiles();
	
	XUB_Fill_ConfigEditBoxes_with_Text()
	XUB_InitialiseProfileNames()
	XUB_Transform_Config() 			-- without recheck, because we did a check at 'validate'
	
	if XUB.CurrentProfile ~= "" then
		XUB.manualChange = 1;
		XUB_ApplyProfile(XUB.CurrentProfile);
	else
		XUB_RemoveButtons();
	end
	
	XUB_ConfigSaveAndExit:Hide();
end

function XUB_TableSort(myTable)

	local mySortTable = {}
	local myResultTable = {}

	for index in pairs(myTable) do 
		table.insert(mySortTable, index);
	end
	table.sort(mySortTable);

	for i = 1,#mySortTable do
		table.insert(myResultTable, myTable[mySortTable[i]]);
	end
	
	return myResultTable
	
end

function XUB_SaveButtons()

	XUB.CustomButtons = XUB_TableSort(myCompleteCustomButtonInfo);
	
	-- build runtime array:
	myCustomButtons_at_runtime = {}
	for i=1,#XUB.CustomButtons do
		if ( XUB.CustomButtons[i][4] == "macro" ) or ( myID ~= 0) then
			table.insert(myCustomButtons_at_runtime, XUB.CustomButtons[i] );
		end
	end
end

function XUB_SaveGroups()

	local myDummyTable = {};
	local myGroups = XUB_SplitTextIntoTable( XUB_ConfigPanel2GroupsEdit:GetText() );
	local myTrim,myItems

	XUB.ConfigGroups = {};

	for i = 1,# myGroups do
		myTrim,myItems = XUB_SplitCommaAndTrim(myGroups[i]);
--		myTrim[4] = myTrim[4] or "";			-- EXCLUDEUNIT
		myDummyTable[myTrim[1].."~~"..string.format("%03d",i)] = myTrim; 	-- Sort by column 1 (=groupName), make them unique by adding the index 'i'
	end
	
	XUB.ConfigGroups = XUB_TableSort(myDummyTable);

end

function XUB_SaveProfiles()

	local myDummyTable = {};
	local mySortColumn = 1;
	local myProfiles = XUB_SplitTextIntoTable( XUB_ConfigPanel3ProfilesEdit:GetText() );
	local myTrim,myItems
	local myConfiguredUnits = {}
	
	XUB.ConfigProfiles = {};

	for i = 1,# myProfiles do
		myTrim,myItems = XUB_SplitCommaAndTrim(myProfiles[i]);
--		myTrim[4] = myTrim[4] or 0;			-- xOffset
--		myTrim[5] = myTrim[5] or 0;			-- yOffset
		myDummyTable[myTrim[1]..myTrim[2].."~~"..string.format("%03d",i)] = myTrim; 	-- Sort by column 1 AND 2 ( = profileName+TargetFrame)
	end

	XUB.ConfigProfiles = XUB_TableSort(myDummyTable);
	
	local myFirstProfile = ""
	if XUB.ConfigProfiles and XUB.ConfigProfiles[1] and XUB.ConfigProfiles[1][1] then
		myFirstProfile = XUB.ConfigProfiles[1][1]
	end
	if myFirstProfile ~= "" then			-- we have a valid profile
		if ( XUB.CurrentProfile == "" ) then		-- if we had no config yet
			
			-- set the CurrentConfig to the first new profile
			XUB.CurrentProfile = myFirstProfile;
			
			-- enable all event because now we have a valid config
			PBFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
			PBFrame:RegisterEvent("PARTY_MEMBERS_CHANGED");
			PBFrame:RegisterEvent("RAID_ROSTER_UPDATE");
			PBFrame:RegisterEvent("LEARNED_SPELL_IN_TAB");
			PBFrame:RegisterEvent("SPELLS_CHANGED");
			PBFrame:RegisterEvent("PLAYER_REGEN_ENABLED");
			PBFrame:RegisterEvent("PLAYER_TALENT_UPDATE");
		elseif ( foundCurrentProfile == nil ) then	-- we have a CurrentConfig, but its profile no longer exists
			XUB.CurrentProfile = myFirstProfile;
		end
	elseif ( foundCurrentProfile == nil ) then	-- no valid config, and additionally the old profile no longer exists
			XUB.CurrentProfile = "";
	end
	
end

function XUB_SplitTextIntoTable(aText)

	local s=1;
	local myLine = 0;
	local myLen = aText:len();
	local mySource = {};

	for i = 1,myLen do
		if ( aText:sub(i,i) == "\n" ) then
			if ( strtrim(aText:sub(s,i-1)) ~= "" ) then
				myLine = myLine + 1
				mySource[myLine] = strtrim(aText:sub(s,i-1));
			end
			s = i+1;
		end
	end

	if ( myLen > 0 ) then
		if ( strtrim(aText:sub(s,myLen)) ~= "" ) then
			myLine = myLine + 1
			mySource[myLine] = strtrim(aText:sub(s,myLen));
		end
	end

	return mySource;
end

function XUB_SplitCommaAndTrim(aText)
	local myTrim = {}
	local myItems = { strsplit(",",aText) };

	for i=1,#myItems do
			myTrim[i] = strtrim(myItems[i]);
	end
	return myTrim,myItems
end

function XUB_UseItemCheck(myItem,myButtonName)
	
	local myItemID = 0;
	local myItemTexture = "Interface\\Icons\\INV_Misc_QuestionMark";
	
	myItemName, myItemLink, _, _, _, _, _, _, _, myItemTexture = GetItemInfo(myItem)

	if myItemLink then
		_, _, _, _, myItemID = string.find(myItemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
		myItemID = tonumber(myItemID) or 0;
		if ( myItemID == 0 ) then
			XUB_ErrorMessage( "XUB: Warning: Use item '" .. CCC(myItem) .. "' is not valid. "  .. CCC( myButtonName ) );
		end
	else
		XUB_ErrorMessage( "XUB: Warning: Use item '" .. CCC(myItem) .. "' is not valid. "  .. CCC( myButtonName ) );
	end

	if ( myItemTexture == nil ) or ( myItemTexture == "" ) then
		myItemTexture = "Interface\\Icons\\INV_Misc_QuestionMark";
	end
	
	return myItemID, myItemTexture;
end	

function XUB_CheckSpell(aSpellName, aButtonName)

	local l_isValidSpell = 1;
	local l_Texture = "Interface\\Icons\\INV_Misc_QuestionMark";
	
	local l_SkillType, l_SpellId = GetSpellBookItemInfo( aSpellName );
	l_SpellId = l_SpellId or 0
	
	if ( l_SkillType == nil ) or ( l_SpellId == 0 ) then
		XUB_ErrorMessage( "XUB: Warning: Spell " .. CCC(aSpellName) .. " cannot be validated . Possible Reasons: typo, dualspec, low level ..." );
	else
		local l_isPassive = IsPassiveSpell(aSpellName);
		if ( l_isPassive) then
			XUB_ErrorMessage( "XUB: Error: " .. CCC(aButtonName) .. " button refers to a passive ability. Spell "  .. CCC( aSpellName ) .. " is passive." );
			l_isValidSpell = 0;
		end
	end

	if l_isValidSpell == 1 then
		_, _, l_Texture = GetSpellInfo( l_SpellId )
		if ( l_Texture == nil ) or ( l_Texture == "" ) then
			l_Texture = "Interface\\Icons\\INV_Misc_QuestionMark";
		end
	end


	return l_isValidSpell,l_SpellId,l_Texture
	
end

function XUB_GetMacro()
	XUB_Config:Hide();

	XUB_SelectMacroReturnScreenLabel:SetText("XUB_Config");
	XUB_SelectMacro:Show();
end

function XUB_C_MakeGroups()
	XUB_Config:Hide();
	XUB_SelectSpellReturnScreenLabel:SetText("XUB_Config");
	XUB_SelectSpell:Show();
end

function XUB_C_MakeProfiles()
	XUB_Config:Hide();
	XUB_MakeProfileReturnScreenLabel:SetText("XUB_Config");
	XUB_MakeProfile:Show();
end

function XUB_ReCheck_ButtonInfo()
-- this function tries to fill in the global_ID and the texture for all buttons
-- Reason: After a spec change (dual spec), some spells become valid
-- but we have no ID and no texture yet. So will fetch them here.

	XUB_GenerateDefaultSpellList()

	local l_Texture = "";
	local myButtonType, myIsValidSpell, myID, myTexture
	
	myCustomButtons_at_runtime = {}
	
	for i=1,#XUB.CustomButtons do
		myButtonType = string.lower(XUB.CustomButtons[i][4]);
		myID = 0;
		if ( myButtonType == "macro" ) then
			-- do no recheck, just add the current line to the runtime array
			table.insert(myCustomButtons_at_runtime, XUB.CustomButtons[i] );
		elseif ( myButtonType == "use" ) then
			-- recheck if the use item is existing now
			XUB.SilentTest = 1;
			myID, myTexture = XUB_UseItemCheck(XUB.CustomButtons[i][5], XUB.CustomButtons[i][3])
			XUB.SilentTest = nil;
		else		-- if it's not 'macro' or 'use' then it must be some kind of spell
			-- recheck the spell
			XUB.SilentTest = 1;
			myIsValidSpell, myID, myTexture = XUB_CheckSpell(XUB.CustomButtons[i][5], XUB.CustomButtons[i][3])
			XUB.SilentTest = nil;
		end
		if ( myID ~= 0) then 	-- if the button is valid  (not invalid)
			XUB.CustomButtons[i][1] = myID 			-- save the globalID
			XUB.CustomButtons[i][2] = myTexture		-- save the texture
			table.insert(myCustomButtons_at_runtime, XUB.CustomButtons[i] ); 		-- add the current line to the runtime array
		end
	end
	
	XUB_Transform_Config()
	
end

function XUB_Transform_Config()

	local l_tempbuttons = {}
	local l_tempgroups = {}
	local l_dummyArray = {}
	local l_profileName, l_groupName, l_buttonName
	
	-- Transform profiles
	XUB.ProfileDescription = {}
	for i=1,#XUB.ConfigProfiles do
		l_profileName = XUB.ConfigProfiles[i][1]
		l_groupName = XUB.ConfigProfiles[i][3]
		l_tempgroups[l_groupName] = 1
		XUB.ProfileDescription[l_profileName] = XUB.ProfileDescription[l_profileName] or {}
		l_dummyArray = {}
		l_dummyArray.profileName = XUB.ConfigProfiles[i][1]
		l_dummyArray.attachUnit = string.lower(XUB.ConfigProfiles[i][2])
		l_dummyArray.groupName = XUB.ConfigProfiles[i][3]
		l_dummyArray.xOffset = XUB.ConfigProfiles[i][4] or 0
		l_dummyArray.yOffset = XUB.ConfigProfiles[i][5] or 0
		--table.insert(XUB.ProfileDescription[l_profileName], XUB.ConfigProfiles[i])
		table.insert(XUB.ProfileDescription[l_profileName], l_dummyArray)
	end

	-- Transform groups
	XUB.GroupDescription = {}
	for i=1,#XUB.ConfigGroups do
		l_groupName = XUB.ConfigGroups[i][1]
		if l_tempgroups[l_groupName] then 			-- only add a group if it is used in a profile, see above
			l_buttonName = XUB.ConfigGroups[i][2]
			l_tempbuttons[l_buttonName] = 1
			XUB.GroupDescription[l_groupName] = XUB.GroupDescription[l_groupName] or {}
			l_dummyArray = {}
			l_dummyArray.groupName = XUB.ConfigGroups[i][1]
			l_dummyArray.buttonName = XUB.ConfigGroups[i][2]
			l_dummyArray.destination = string.lower(XUB.ConfigGroups[i][3])
			l_dummyArray.excludeUnit = XUB.ConfigGroups[i][4] or ""
			--table.insert(XUB.GroupDescription[l_groupName], XUB.ConfigGroups[i])
			table.insert(XUB.GroupDescription[l_groupName], l_dummyArray)
		end
	end
	
	-- Transform all buttons in one single array (standard buttons AND custom buttons !)
	XUB.ButtonDescription = {}
	for i=1,#XUB.StandardButtons do
		l_buttonName = XUB.StandardButtons[i][3]
		if l_tempbuttons[l_buttonName] then 			-- only add a button if it is used in a group, see above
			XUB.ButtonDescription[l_buttonName] = XUB.ButtonDescription[l_buttonName] or {}
			l_dummyArray = {}
			l_dummyArray.globalID = XUB.StandardButtons[i][1]
			l_dummyArray.texture = XUB.StandardButtons[i][2]
			l_dummyArray.buttonName = XUB.StandardButtons[i][3]
			l_dummyArray.buttonType = string.lower(XUB.StandardButtons[i][4])
			l_dummyArray.action = XUB.StandardButtons[i][5]
			l_dummyArray.condition = XUB.StandardButtons[i][6] or ""
			--table.insert(XUB.ButtonDescription[l_buttonName], XUB.StandardButtons[i])
			table.insert(XUB.ButtonDescription[l_buttonName], l_dummyArray)
		end
	end
	for i=1,#myCustomButtons_at_runtime do
		l_buttonName = myCustomButtons_at_runtime[i][3]
		if l_tempbuttons[l_buttonName] then 			-- only add a button if it is used in a group, see above
			XUB.ButtonDescription[l_buttonName] = XUB.ButtonDescription[l_buttonName] or {}
			l_dummyArray = {}
			l_dummyArray.globalID = myCustomButtons_at_runtime[i][1]
			l_dummyArray.texture = myCustomButtons_at_runtime[i][2]
			l_dummyArray.buttonName = myCustomButtons_at_runtime[i][3]
			l_dummyArray.buttonType = string.lower(myCustomButtons_at_runtime[i][4])
			l_dummyArray.action = myCustomButtons_at_runtime[i][5]
			l_dummyArray.condition = myCustomButtons_at_runtime[i][6] or ""
			--table.insert(XUB.ButtonDescription[l_buttonName], myCustomButtons_at_runtime[i])
			table.insert(XUB.ButtonDescription[l_buttonName], l_dummyArray)
		end
	end
	
	l_dummyArray = {}	

end

function XUB_CDP_OnShow()
	XUB_Config:Hide();
	XUB_SetBackground(XUB_ChooseDefaultProfiles)
	UIDropDownMenu_SetSelectedID(XUB_ChooseDefaultProfilesSpec1,0,0)
	UIDropDownMenu_SetSelectedID(XUB_ChooseDefaultProfilesSpec2,0,0)
	XUB_ChooseDefaultProfilesSpec1Text:SetText(XUB.Profile_for_Spec[1])
	XUB_ChooseDefaultProfilesSpec2Text:SetText(XUB.Profile_for_Spec[2])
end

function XUB_CDP_Save()
	XUB.Profile_for_Spec = TempProfile_for_Spec;
	XUB_ShowMessage("XUB: Profile "..CCC(XUB.Profile_for_Spec[1]).." is now default profile for spec 1")
	XUB_ShowMessage("XUB: Profile "..CCC(XUB.Profile_for_Spec[2]).." is now default profile for spec 2")
end

function XUB_SetSpecProfile(myFrame,mySpec,userChoice,selectedProfile)
	UIDropDownMenu_SetSelectedID(myFrame, userChoice, 0);
	if selectedProfile ~= "--- no default profile ---" then
		TempProfile_for_Spec[mySpec] = selectedProfile;
	else
		TempProfile_for_Spec[mySpec] = "";
	end
end

function XUB_ChooseDefaultProfiles_OnShow(myFrame,mySpec)

	local info = {}
	info.text = "--- no default profile ---"
	info.func = function() XUB_SetSpecProfile(myFrame,mySpec,1,"--- no default profile ---"); end
	UIDropDownMenu_AddButton(info); 
	for i = 1,#XUB.profileNames do
		info.text = XUB.profileNames[i]
		info.func = function() XUB_SetSpecProfile(myFrame,mySpec,i+1,XUB.profileNames[i]); end
		UIDropDownMenu_AddButton(info); 
	end
end
