﻿local addonName, XUB = ...;

--[[
internals of profiles:

Profile[1]: profile name
Profile[2]: target frame
Profile[3]: group name
Profile[4]: X-offset (optional)
Profile[5]: Y-offset (optional)

--]]

local l_name = 1;
local l_target = 2;
local l_groupName = 3;
local l_x_offset = 4;
local l_y_offset = 5;

local l_OldProfileName = "~first~profile~";

local XUB_MP_SelectedGroup = 0;
local myGroupNames = {};

--
-- function identifies the first 9 groups in a players group box
-- and initializes the rest of the form.
function XUB_MP_OnShow()

--  XUB_MakeProfile:SetAlpha(1);
	XUB_MP_SelectedGroup = 0;
	XUB_MakeProfile_ProfileName:SetText("");
	XUB_MakeProfileListLabel0Label:SetText("*** NO SELECTION ***");
	XUB_SetBackground(XUB_MakeProfile);
	XUB_MakeProfileTextPanelGroupEdit:SetText("");

	XUB_GenerateGroupList()

	XUB_MakeProfile:Raise();

	XUB_MakeProfileListSliderSlider:SetValue(1);
	
	local myMaxValue = #myGroupNames

	if myMaxValue < 10 then
	-- less then 10 buttons
		XUB_MakeProfileListSliderSlider:SetMinMaxValues(1,1);
		XUB_MakeProfileListSliderSlider:Disable();
	else
		XUB_MakeProfileListSliderSlider:SetMinMaxValues(1,myMaxValue-8);
		XUB_MakeProfileListSliderSlider:Enable();
	end

	XUB_MP_ShowGroups(1);
end

function XUB_GenerateGroupList()

	local currentGroupName, existingGroupName
	local myGroupEditboxContent = XUB_SplitTextIntoTable( XUB_ConfigPanel2GroupsEdit:GetText() );
	local already_exists = nil;
	
	myGroupNames = { "CR" }

	for i = 1,# myGroupEditboxContent do 					-- iterate through all the lines of the group editbox
		currentGroupName = strsplit(",",myGroupEditboxContent[i])
		
		already_exists = nil;
		-- check if the current groupname already exists
		for ii, existingGroupName in ipairs(myGroupNames) do
			if strtrim(existingGroupName) == strtrim(currentGroupName) then
				already_exists = 1
				break
			end
		end
		
		-- take each groupname only once
		if (not already_exists) and (currentGroupName ~= "" ) then
			table.insert(myGroupNames,currentGroupName);
		end
	end
end

function XUB_MP_ShowGroups(aIndex)

	local myMaxValue = #myGroupNames;
	if myMaxValue > 9 then
		myMaxValue = 9;
	end
	
	for i=1,myMaxValue do
		XUB_MP_ShowAGroup(i,aIndex+i-1);
	end
end

function XUB_MP_ShowAGroup(aPosition,aEntryID)

	local myLabel = getglobal("XUB_MakeProfileListLabel" .. aPosition .. "Label" );
	local myTexture = getglobal("XUB_MakeProfileListTexture" .. aPosition .. "Texture" );

	local mySelectedLabel = getglobal("XUB_MakeProfileListLabel0Label" );
	
	myTexture:SetTexture( 1,1,1 );
	myLabel:SetText( myGroupNames[aEntryID] );
	
	if ( aEntryID == XUB_MP_SelectedGroup ) then
		mySelectedLabel:SetText( myGroupNames[aEntryID] );

		myTexture:SetVertexColor(0.3,0.3,0.3);
	else
		myTexture:SetVertexColor(1,1,1);
	end
end


function XUB_MP_AppendGroup(aEntryID)

	local myProfileName = XUB_MakeProfile_ProfileName:GetText();
	local myEditboxText = XUB_MakeProfileTextPanelGroupEdit:GetText();
	
	if ( myProfileName == "" ) then
		myProfileName = "UnknownProfile";
	end
	
	local myNewText = myProfileName;
	local myTargetFrame = strtrim(XUB_MakeProfileTargetText:GetText());
	if ( myEditboxText ) and ( myEditboxText ~= "" ) then
		if myProfileName..myTargetFrame ~= l_OldProfileName then
			myNewText = "\n\n" .. myProfileName;
		else
			myNewText = "\n" .. myProfileName;
		end
	end
	XUB_MakeProfileTextPanelGroupEdit:SetText( myEditboxText .. myNewText .. "," .. myTargetFrame .. "," ..  myGroupNames[aEntryID] );
	
	l_OldProfileName = myProfileName..myTargetFrame;
end

function XUB_MP_OnValueChanged()
	local myItemNbr = XUB_MakeProfileListSliderSlider:GetValue();
	XUB_MP_ShowGroups(myItemNbr);
end

function XUB_MP_SetGroup(aPosition)
	XUB_MP_SelectedGroup = XUB_MakeProfileListSliderSlider:GetValue() + aPosition - 1;
	XUB_MP_OnValueChanged();

	XUB_MP_AppendGroup(XUB_MP_SelectedGroup);
end

function XUB_MakeProfile_OnClickOK()
	XUB_MakeProfile:Hide();
--	XUB_Config_AddNewContentToGroups(myNewContentForGroups);
	XUB_Config_AddNewContentToProfiles(XUB_MakeProfileTextPanelGroupEdit:GetText());
	local SCR = getglobal(XUB_MakeProfileReturnScreenLabel:GetText());
	SCR:Show();
end

function XUB_MakeProfile_OnClickCancel()
	XUB_MakeProfile:Hide();
	local SCR = getglobal(XUB_MakeProfileReturnScreenLabel:GetText());
	SCR:Show();
end

function XUB_Generate_Group_Tooltip(myButton)

	XUB_GameTooltip:SetOwner(myButton,"ANCHOR_BOTTOMRIGHT");
	XUB_GameTooltip:AddLine(myButton.ToolTip,0,1,1,0);
end

