﻿local addonName, XUB = ...;

XUB.time_for_blink = 0;

XUB.AllBuffs2 = {}
XUB.AllDebuffTypes = {}

--
-- Performs a periodic update of the buttons according to XUB.UPDATERATE which defaults to 2 seconds
--
function XUB_OnUpdate(self,elapsed)

--    XUB_CheckProfiling();

	-- First IF is only processed until the initialisation is complete. After that, it is never run again
	if ( XUB.MakeInitialisation ) then
		-- disregard the first X OnUpdate cycles, because 'elapsed' is sometimes faulty at the beginning
		if (XUB.DismissFirstOnUpdateCycles) then
			XUB.DismissFirstOnUpdateCycles = XUB.DismissFirstOnUpdateCycles-1
			if XUB.DismissFirstOnUpdateCycles < 0 then
			XUB.DismissFirstOnUpdateCycles=nil;
			end
			return
		end
		-- after these first cycles, wait for X seconds (X = XUB.CountDown_before_checking_for_changes, which is 5 at the start of the addon)
		if ( XUB.MakeInitialisation >= 0 ) then
			XUB.MakeInitialisation = XUB.MakeInitialisation - elapsed;
			if ( XUB.MakeInitialisation < 0 ) then
				XUB_DoFirstInitialisation();
				XUB.MakeInitialisation = nil;                                    
				XUB.CountDown_before_checking_for_changes = -1;
			end    
		end
		return;   -- prevents running updates until the initialisation is complete
	end

	if ( XUB.Busy == 1 ) then        -- true while we are checking some changes or making buttons
		return;
	end

	if ( XUB.CountDown_before_checking_for_changes >= 0 ) then
		XUB.CountDown_before_checking_for_changes = XUB.CountDown_before_checking_for_changes - elapsed;
		if ( XUB.CountDown_before_checking_for_changes < 0 ) then  -- Check for the following changes, and applies them if necessary:
			XUB_Check_for_Changes();     -- party/raid changes | newly learned/unlearned spells | profile change (user driven)
			XUB.CountDown_before_checking_for_changes = -1;
		end
	end

	XUB.NextUpdate = XUB.NextUpdate - elapsed;
	if ( XUB.NextUpdate > 0 ) then
		return;
	end

	XUB.NextUpdate = XUB.UPDATERATE;        -- When do we come by next?
	_,XUB.time_for_blink = math.modf(XUB.time_for_blink + XUB.BLINKRATE);

	-- if there is a button that is currently mouse overed and in the State COOLDOWN then UpdateTheButtonsTooltip
	if ( XUB.MouseOveredButton ) then
		if ( XUB.MouseOveredButton.is_STATE_COOLDOWN ) then
			XUB_UpdateTooltip(XUB.MouseOveredButton);
		end
	end
	
	--XUB.DEBUG = 0;
	XUB_UpdateButtons();
	--XUB.DEBUG = 1;

	XUB.HealthLeftTarget = 1000000;    -- The approximate % to damage ratio

end

--
-- Updates buttons on the screen according to unit.  To a maximum of the number
-- of units on the screen or the XUB.QUEUESIZE (which ever is smaller).
--
function XUB_UpdateButtons()

	XUB.MaxToUpdate = math.min( #XUB.UPDATEUNITS,XUB.QUEUESIZE );

	for i = 1,XUB.MaxToUpdate do

		if ( XUB.QUEUEPTR > #XUB.UPDATEUNITS ) then
			XUB.QUEUEPTR = 1;
		end

		XUB_UpdateUnitButtons2(XUB.UPDATEUNITS[XUB.QUEUEPTR]);
		XUB.QUEUEPTR = XUB.QUEUEPTR + 1;
	end
end


-- function updates all of the buttons that use the specified aUnit as destination unit
function XUB_UpdateUnitButtons2(aUnit)

	if ( aUnit ~= "float" ) then
	-- Check all Buffs and Debuffs of the current Unit
	XUB_CHECK_BUFF_DEBUFF_STATE2(aUnit);
	end

	local myButton
	local buttonCounter = 1;          -- Main counter, used for cycling through all buttons for current unit
	local timeLeft = -1;
	
	local myPreviousState
	local was_STATE_INVALID

	while ( 1 == 1 ) do

		--Get a SINGLE Button to work with it
		myButton = getglobal( XUB_GenerateButtonName(aUnit,buttonCounter) );
		--XUB_ShowMessage( "TESTING '" .. XUB_GenerateButtonName(aUnit,buttonCounter) .. "'");

		-- If we encounter a nil, it means we don't need to test any more buttons on current unit.
		if ( myButton == nil ) then
			return;
		-- button is disbled because he is "removed"
		elseif myButton.isActive==0 then
			return;
		end

		-- Next line can be enabled if you really need to see which units/buttons get updated. May generate a lot of spam. Be warned.
--        if ( XUB.DEBUG > 0 ) then XUB_Show_DebugBlock1(myButton, buttonCounter, aUnit);           end

		myPreviousState = myButton.State;
		was_STATE_INVALID = myButton.is_STATE_INVALID;

		XUB_CHECK_SINGLE_BUTTON(myButton)

		myButton.State =  0
		--if myButton.is_STATE_ENABLED    then myButton.State = myButton.State + 0; end
		if myButton.is_STATE_HEALTHOFF  then myButton.State = myButton.State + 1; end
		if myButton.is_STATE_BUFFED     then myButton.State = myButton.State + 2; end
		if myButton.is_STATE_DIMMED     then myButton.State = myButton.State + 4; end
		if myButton.is_STATE_COOLDOWN   then myButton.State = myButton.State + 8; end
		if myButton.is_STATE_NO_ITEMS   then myButton.State = myButton.State + 16; end
		if myButton.is_STATE_OUTOFMANA  then myButton.State = myButton.State + 32; end
		if myButton.is_STATE_OUTOFRANGE then myButton.State = myButton.State + 64; end
		if myButton.is_STATE_INVALID    then myButton.State = myButton.State + 128; end
		
		if ( myButton.Blink == 1 ) and ( myButton.State < 1 ) then --if the button should blink whenever he is fully enabled
			XUB_HandleBlinking(myButton);
		elseif ( myButton.SpecialLastSeconds ) then       -- if current Button should handle the last seconds differently
			if ( myButton.LastsTill ) then                -- if current Button has a remaining time
				timeLeft = myButton.LastsTill - GetTime() 
				if ( timeLeft < tonumber(myButton.SpecialLastSeconds) ) then
					if ( myButton.isDebuff ) and ( myButton.State < 4 ) then    -- disable the button in the last X seconds
						myButton.State=myButton.State + 128;
						myButton.is_STATE_INVALID = 1;
					else    
						if ( myButton.Blink == 1 ) then
								XUB_HandleBlinking(myButton);
						elseif ( myButton.is_STATE_BUFFED ) and ( myButton.State < 4 ) then    -- enable the button in the last X seconds
								myButton.State=0;
								myButton.is_STATE_BUFFED = nil;
						end
					end
				end
			end
		end

		-- Countdown for buttons in cooldown must be handled no matter if the button state did change or not
		if ( myButton.is_STATE_COOLDOWN ) then
			XUB_Set_CountdownText(myButton);
		end

		if myButton.is_STATE_INVALID and was_STATE_INVALID then
			-- Do nothing, the button was hidden and is still hidden
		else
			if ( myButton.State ~= myPreviousState ) then
				if ( XUB.DEBUG > 0 ) then   XUB_Show_DebugBlock2(myButton, buttonCounter, aUnit, myPreviousState );          end

				--now then lets do some real action with the current button
				XUB_CHANGE_SINGLE_BUTTON(myButton);
			end
		end
		if ( XUB.DEBUG > 0 ) and ( myButton.is_STATE_BUFFED ) and ( XUB.MouseOveredButton ) and ( XUB.MouseOveredButton == myButton ) then  XUB_UpdateTooltip(XUB.MouseOveredButton);   end

		buttonCounter = buttonCounter + 1;
	end

end


function XUB_GetWeaponEnchantName(aSlot)


	XUB_WeaponTooltip:ClearLines();
	XUB_WeaponTooltip:SetInventoryItem("player", aSlot);

	local WeaponBuffText, WeaponBuffName, brack;

	for i=1, XUB_WeaponTooltip:NumLines(), 1 do
		WeaponBuffText = getglobal("XUB_ParseToolTipTextLeft"..i):GetText();
		WeaponBuffName = strmatch(WeaponBuffText, "^.+ %(%d+%s+.+%)$");
		if ( WeaponBuffName ) then
			break;
		end
	end

	if ( WeaponBuffName ) then
		brack = WeaponBuffName:find(" %(");
		if ( brack == nil ) then
			return WeaponBuffName;
		else
			return WeaponBuffName:sub(1,brack-1);
		end
	else
		return "nil";
	end
end


function XUB_CHECK_BUFF_DEBUFF_STATE2(bUnit)

	local name, rank, texture, count, duration, expiryTime, debuffType, unitCaster, isStealable, shouldConsolidate, GlobalSpellId;
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges, hasThrownEnchant, thrownExpiration, thrownCharges;
	local buffCounter
	local l_self = 1
	local WB_name = ""
	
	--XUB.AllBuffs2 = {}
	table.wipe(XUB.AllBuffs2)
	--XUB.AllDebuffTypes = {}
	table.wipe(XUB.AllDebuffTypes)

	if ( bUnit == "player" ) then
		hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges, hasThrownEnchant, thrownExpiration, thrownCharges = GetWeaponEnchantInfo();
		if ( hasMainHandEnchant ) then
			WB_name = string.lower(XUB_GetWeaponEnchantName(GetInventorySlotInfo("MainHandSlot")))
			XUB.AllBuffs2[WB_name] = { ["name"] = WB_name,["expiryTime"] = mainHandExpiration, ["debuffType"] = "", ["selfCast"] = 2, ["GlobalSpellId"] = 0 }
		end
		if ( hasOffHandEnchant ) then
			WB_name = string.lower(XUB_GetWeaponEnchantName(GetInventorySlotInfo("SecondaryHandSlot")))
			XUB.AllBuffs2[WB_name] = { ["name"] = WB_name,["expiryTime"] = offHandExpiration, ["debuffType"] = "", ["selfCast"] = 2, ["GlobalSpellId"] = 0 }
		end
		if ( hasThrownEnchant ) then
			WB_name = string.lower(XUB_GetWeaponEnchantName(GetInventorySlotInfo("RangedSlot")))
			XUB.AllBuffs2[WB_name] = { ["name"] = WB_name,["expiryTime"] = thrownExpiration, ["debuffType"] = "", ["selfCast"] = 2, ["GlobalSpellId"] = 0 }
		end
	end

	-- name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, GlobalSpellId = UnitAura("unit", [index] or ["name", "rank"][, "filter"])
	-- Filter: HELPFUL HARMFUL PLAYER RAID CANCELABLE NOT_CANCELABLE

	-- Get the Buffs on the Unit
	buffCounter=1;
	name, rank, texture, count, debuffType, duration, expiryTime, unitCaster, isStealable, shouldConsolidate, GlobalSpellId = UnitAura(bUnit, buffCounter, "HELPFUL");
	while ( name ~= nil ) do
		name = string.lower(name);
		if unitCaster == "player" then    l_self=2; else l_self=1; end
		XUB.AllBuffs2[name] = { ["name"] = name,["expiryTime"] = expiryTime, ["debuffType"] = debuffType, ["selfCast"] = l_self, ["GlobalSpellId"] = GlobalSpellId }

		buffCounter=buffCounter+1;
		name, rank, texture, count, debuffType, duration, expiryTime, unitCaster, isStealable, shouldConsolidate, GlobalSpellId = UnitAura(bUnit, buffCounter, "HELPFUL");
	end

	-- Get the Debuffs on the Unit
	buffCounter=1;
	name, rank, texture, count, debuffType, duration, expiryTime, unitCaster, isStealable, shouldConsolidate, GlobalSpellId = UnitAura(bUnit, buffCounter, "HARMFUL");

	while ( name ~= nil ) do
		name = string.lower(name);
		if unitCaster == "player" then    l_self=2; else l_self=1; end
		XUB.AllBuffs2[name] = { ["name"] = name, ["expiryTime"] = expiryTime, ["debuffType"] = debuffType, ["selfCast"] = l_self, ["GlobalSpellId"] = GlobalSpellId }
		if debuffType then
			debuffType=string.lower(debuffType)
			XUB.AllDebuffTypes[debuffType] = { ["expiryTime"] = expiryTime }
		end

		buffCounter=buffCounter+1
		name, rank, texture, count, debuffType, duration, expiryTime, unitCaster, isStealable, shouldConsolidate, GlobalSpellId = UnitAura(bUnit, buffCounter, "HARMFUL");
	end
end


function XUB_CHECK_for_GLOBAL_DIM(myButton)

	if ( XUB.USE_GLOBALDIM == 1 ) then                      -- if Global Dim is enabled
		if ( XUB.GLOBALDIM_active == 1 ) then               -- if Global Dim is currently active
			myButton.is_STATE_DIMMED = 1;
		end
	end
end


function XUB_CHECK_SINGLE_BUTTON(myButton)

	local myPreviousCooldown = myButton.MyCooldown;
	local myPreviousCooldownState = myButton.is_STATE_COOLDOWN;

	-- We start assuming everything is OK.
--    myButton.is_STATE_ENABLED    = 1;
	myButton.is_STATE_HEALTHOFF  = nil;
	myButton.is_STATE_BUFFED     = nil;
	myButton.is_STATE_DIMMED     = nil;
	myButton.is_STATE_COOLDOWN   = nil;
	myButton.is_STATE_NO_ITEMS   = nil;
	myButton.is_STATE_OUTOFMANA  = nil;
	myButton.is_STATE_OUTOFRANGE = nil;
	myButton.is_STATE_INVALID    = nil;
	myButton.MyCooldown          = 0;
	myButton.LastsTill           = nil;

	--XUB_ShowMessage("UnitCheck: "..CCC(UnitName(myButton.DestinationUnit)));
	if ( myButton.Bar == nil ) then                                 -- does the button have a 'unit' where it is attached to? (FLOAT counts as unit)
		myButton.is_STATE_INVALID = 1;
		return;
	elseif ( UnitExists(myButton.DestinationUnit) == nil ) then     -- does the destination unit exist ?
		myButton.is_STATE_INVALID = 1;
		return;
	elseif ( UnitName(myButton.DestinationUnit) == nil ) then       -- do we get a name for the destination unit ?
		myButton.is_STATE_INVALID = 1;
		return;
	elseif ( UnitInVehicle(myButton.DestinationUnit) ) then       -- if the unit is in a vehicle
		myButton.is_STATE_INVALID = 1;
		return;
--    elseif ( myButton.Bar ~= "float" ) then             -- do not perform the next checks for the float bar, since they can never work
--        if ( UnitExists(myButton.Bar) == nil ) then                 -- does the unit where the button is attached to exist ?
--            myButton.is_STATE_INVALID = 1;
--            return;
--        elseif ( UnitName(myButton.Bar) == nil ) then               -- do we get a name of the unit where the button is attached to
--            myButton.is_STATE_INVALID = 1;
--            return;
--        end
	end

	if ( UnitIsDeadOrGhost("player") ) then
		myButton.is_STATE_INVALID = 1;
		return;
	end

	if ( UnitIsDeadOrGhost(myButton.DestinationUnit) ) then
		if myButton.Condition == "death" then
			-- no return, because we need to continue. Unit could still be out of range or out of mana etc.
		else
			myButton.is_STATE_INVALID = 1;
			return;
		end
	end

	if ( myButton.ExcludeUnits ~= "" ) then 		-- if there are Exclude Units declared for the button
		if UnitClass(myButton.DestinationUnit) and string.find( myButton.ExcludeUnits .. "/" , UnitClass(myButton.DestinationUnit) .. "/" ) then
			myButton.is_STATE_INVALID = 1;
			return;
		elseif UnitCreatureType(myButton.DestinationUnit) and string.find( myButton.ExcludeUnits .. "/" , UnitCreatureType(myButton.DestinationUnit) .. "/" ) then
			myButton.is_STATE_INVALID = 1;
			return;
		else
			if UnitIsEnemy("player",myButton.DestinationUnit) then 		-- if my destination unit is an enemy
				if string.find( string.lower(myButton.ExcludeUnits .. "/") , "hostile/" ) then -- and hostile units are excluded
					myButton.is_STATE_INVALID = 1;
					return;
				end
			else 														-- if my destination unit is a friendly unit
				if string.find( string.lower(myButton.ExcludeUnits .. "/") , "friendly/" ) then -- and friendly units are excluded
					myButton.is_STATE_INVALID = 1;
					return;
				end
			end
		end
	end

	if ( myButton.isMacro == 1 ) then
		if myButton.DestinationUnit ~= "player" then
			local inRange = UnitInRange( myButton.DestinationUnit );
			--XUB_ShowMessage("inRange: "..CCC(inRange))            
			if ( inRange == nil ) then
				myButton.is_STATE_INVALID = 1;
				--myButton.is_STATE_OUTOFRANGE = 1;
			elseif ( inRange == 0 ) then
				myButton.is_STATE_OUTOFRANGE = 1;
			end
		else
			-- Do Nothing -- a macro without an explicit destination unit is always enabled
		end
	elseif ( myButton.isUse ) then

		-- Check if the target is in Range ?
		if ( myButton.DestinationUnit ~= "player" ) then                           -- no range check needed for player
			local inRange = IsItemInRange( myButton.global_ID , myButton.DestinationUnit );
			if ( inRange == nil ) then
				myButton.is_STATE_INVALID = 1;
				return;
			elseif ( inRange == 0 ) then
				myButton.is_STATE_OUTOFRANGE = 1;
			end
		end

		-- Check if a buff condition is given for a certain item
		if myButton.Condition then
			for i = 1,#myButton.Condition do
				if myButton.Condition[i].inverted then 		--if "!"
					myButton.is_STATE_BUFFED = 1;
					if XUB.AllDebuffTypes[myButton.Condition[i].name] then
						myButton.is_STATE_BUFFED = nil;
						myButton.LastsTill = XUB.AllDebuffTypes[myButton.Condition[i].name].expiryTime
					end
				else
					if XUB.AllBuffs2[myButton.Condition[i].name] then
						myButton.is_STATE_BUFFED = 1;
						myButton.LastsTill = XUB.AllBuffs2[myButton.Condition[i].name].expiryTime
					end
				end
			end
		end
		
		-- Check if the item is on Cooldown
		local start, duration, enable = GetItemCooldown(myButton.global_ID);
		if enabled == 0 then
			-- cooldown triggered, but countdown not started yet due to spell still active
			XUBShowDebugMessage("Spell is currently active, use it and wait " .. duration .. " seconds for the next one.");
			myButton.is_STATE_COOLDOWN = 1;
			myButton.CooldownFrame.text:SetText("")
			myButton.ShowCountdown = nil;
		elseif ( start > 0 and duration > 0) then
			-- cooldown active
			myButton.is_STATE_COOLDOWN = 1;
			myButton.CooldownFrame.text:SetText("") 		-- exact value will be set later
			local remaining = (duration - (GetTime() - start));
			myButton.MyCooldown = remaining;
			if remaining > myPreviousCooldown then                  -- fixes the fact that some spells only report the global CD first
				if XUB.USE_BUILTIN_COOLDOWN == 1 then
					CooldownFrame_SetTimer(myButton.CooldownFrame, GetTime(), remaining, 1);
				end
				if (duration > 1.5) then
					myButton.ShowCountdown = 1
				end
			end
		else
		 -- no cooldown
		 	if myPreviousCooldownState then 		-- if the button was in Cooldown in the last cycle
				CooldownFrame_SetTimer(myButton.CooldownFrame, 0, 0, 0);
				myButton.ShowCountdown = nil;
				myButton.CooldownFrame.text:SetText("")
			end
		end

		XUB_CHECK_for_GLOBAL_DIM(myButton);

		-- Check if the item is usable at the moment ?
		local myUsable = IsUsableItem( myButton.global_ID );
		if ( myUsable == nil ) then
			myButton.is_STATE_NO_ITEMS = 1;
			return;
		end

	elseif ( myButton.isCastType ) then

		-- check if button is in cooldown
		local start, duration, enabled = GetSpellCooldown(myButton.global_ID);
		if enabled == 0 then
			-- cooldown triggered, but countdown not started yet due to spell still active
			XUBShowDebugMessage("Spell is currently active, use it and wait " .. duration .. " seconds for the next one.");
			myButton.is_STATE_COOLDOWN = 1;
			myButton.CooldownFrame.text:SetText("")
			myButton.ShowCountdown = nil;
		elseif ( start > 0 and duration > 0) then
			-- cooldown active
			myButton.is_STATE_COOLDOWN = 1;
			myButton.CooldownFrame.text:SetText("") 		-- exact value will be set later
			local remaining = (duration - (GetTime() - start)) - 0.15; 		 			-- 0.15: very experimental nonsense ^^
			myButton.MyCooldown = remaining;
			if remaining > myPreviousCooldown then                  -- fixes the fact that some spells only report the global CD first
				if XUB.USE_BUILTIN_COOLDOWN == 1 then
					CooldownFrame_SetTimer(myButton.CooldownFrame, GetTime(), remaining, 1);
				end
				if (duration > 1.5) then
					myButton.ShowCountdown = 1
				end
			end
		else
		 -- no cooldown
		 	if myPreviousCooldownState then 		-- if the button was in Cooldown in the last cycle
				CooldownFrame_SetTimer(myButton.CooldownFrame, 0, 0, 0);
				myButton.ShowCountdown = nil;
				myButton.CooldownFrame.text:SetText("")
			end
		end

--        if (myButton.is_STATE_INVALID) or (myButton.is_STATE_COOLDOWN) then  return;   end
		if ( myButton.isBuff ) then
			XUB_CheckBuffCondition(myButton)
		elseif ( myButton.isDebuff ) then
			XUB_CheckDebuffCondition(myButton)
		elseif ( myButton.isSpell ) then
			XUB_CheckSpellCondition(myButton)
		elseif ( myButton.isHeal ) then
			XUB_CheckHealthCondition(myButton)
		elseif ( myButton.isAlways ) then
			-- do not check
		else
			XUB_ShowMessage("XUB: Error. Unrecognized Button, some kind of spell, but subclass is unknown." .. CCC(myButton.FullSpellName));
		end

		if (myButton.is_STATE_INVALID) then  return;   end

		XUB_CHECK_for_GLOBAL_DIM(myButton);

		-- Does the player have sufficient Mana/Rage/Energy and skill to cast the spell?
		local myUsable, myNoMana = IsUsableSpell(myButton.global_ID);
		if (not myUsable) then
			--Spell cannot be casted
			if (not myNoMana) then
				local really_notusable = 1;				
				--TAYEDAEN's QUICKFIX for strange behaviour of some spells (Blizzard):
				for i = 1, #XUB.IsUsableSpell_Exceptions do
					if string.find(myButton.FullSpellName, XUB.IsUsableSpell_Exceptions[i]) then
						--XUB_ShowDebugMessage( CCC(XUB.IsUsableSpell_Exceptions[i]) .. " is always enabled because he is included in XUB.IsUsableSpell_Exceptions.lua"  );
						really_notusable = nil;
						break;
					end
				end
				--Spell cannot be casted (but not because of Mana reasons)
				if really_notusable then
					myButton.is_STATE_INVALID = 1;
				end
			else
				--not enough mana to cast the spell
				myButton.is_STATE_OUTOFMANA = 1;
			end
		end

		if (myButton.is_STATE_INVALID) then  return;   end

		-- Disable the button if it is not in range or valid for the unit or if that unit is in a duel
		if ( myButton.DestinationUnit ~= "player" ) then
			local inRange = IsSpellInRange(myButton.FullSpellName,myButton.DestinationUnit)
			if ( inRange == nil ) then
				-- That means either (from WoWWiki)
				-- 1. The spell cannot be cast on the unit. i.e. attempting to check range using 'Frostbolt' on a party member will always return 'nil', similarly testing 'Heal' against an enemy target will also return nil.
				-- 2. If the unit is not 'visible' (see API UnitIsVisible) then you will get a nil, and not a '0' as you might expect.
				-- 3. The unit doesnt exist (e.g. 'target' when you have nothing targetted)
				-- 4. The current player does not know the spell (so you cannot use 'Heal' to test 40 yard range for anyone other than a priest)
				local really_OutOfRange = 1;
				for i = 1, #XUB.IsSpellInRange_Exceptions do
					if string.find(myButton.FullSpellName, XUB.IsSpellInRange_Exceptions[i]) then
						--XUB_ShowDebugMessage( CCC(XUB.IsSpellInRange_Exceptions[i]) .. " is always enabled because he is included in XUB.IsSpellInRange_Exceptions.lua"  );
						really_OutOfRange = nil;
					end
				end
				--Spell cannot be casted (but not because of Mana reasons)
				if really_OutOfRange then
					myButton.is_STATE_INVALID = 1;
				end
			elseif ( inRange == 0 ) then
				-- means the target is not in Range
				myButton.is_STATE_OUTOFRANGE = 1;
			end
		end
	end
end


function XUB_CHANGE_SINGLE_BUTTON(myButton)

	-- Set the defaults for all buttons
	myButton:GetNormalTexture():SetVertexColor(1,1,1);
	myButton:GetHighlightTexture():SetAlpha( 0 );  --make the button look like its unusable if mouse overed

	if myButton.is_STATE_INVALID then
		myButton:SetAlpha( XUB.INVALID_ALPHA );
		--XUB_ShowMessage("myButton.FullSpellName: " .. CCC(myButton.FullSpellName));
	elseif myButton.is_STATE_OUTOFRANGE then
		if ( myButton.is_STATE_BUFFED ) and ( XUB.HEALACTION == "Show as Disabled" ) then
						myButton:SetAlpha( XUB.INVALID_ALPHA );
		elseif ( XUB.COLORUSAGE ~= "Transparency" ) then
			myButton:GetNormalTexture():SetVertexColor(XUB.COLORS["c_range"].r, XUB.COLORS["c_range"].g,XUB.COLORS["c_range"].b);
			myButton:SetAlpha( XUB.ENABLE_ALPHA * 0.7 );
		else
			myButton:SetAlpha( XUB.OUTOFRANGE_ALPHA );
		end
	elseif myButton.is_STATE_OUTOFMANA then
		if ( myButton.is_STATE_BUFFED ) and ( XUB.HEALACTION == "Show as Disabled" ) then
						myButton:SetAlpha( XUB.INVALID_ALPHA );
		elseif ( XUB.COLORUSAGE ~= "Transparency" ) then
			myButton:GetNormalTexture():SetVertexColor(XUB.COLORS["c_outofmana"].r, XUB.COLORS["c_outofmana"].g,XUB.COLORS["c_outofmana"].b);
			myButton:SetAlpha( XUB.ENABLE_ALPHA * 0.9 );
		else
			myButton:SetAlpha( XUB.NOMANA_ALPHA );
		end
	elseif myButton.is_STATE_NO_ITEMS  then
		myButton:SetAlpha( XUB.NOMANA_ALPHA );
	else    -- the rest of the states is NON-exclusive !!

		myButton:GetNormalTexture():SetVertexColor(1,1,1);
		myButton:SetAlpha( XUB.ENABLE_ALPHA );

--        if myButton.is_STATE_ENABLED   then
--            myButton:GetHighlightTexture():SetAlpha( 1 );       --makes the button react like it can be clicked ;-)
--        end

		if myButton.is_STATE_DIMMED    then
			if ( XUB.COLORUSAGE ~= "Transparency" ) then
				myButton:GetNormalTexture():SetVertexColor(XUB.COLORS["c_globaldim"].r, XUB.COLORS["c_globaldim"].g,XUB.COLORS["c_globaldim"].b);
				myButton:SetAlpha( XUB.ENABLE_ALPHA );
			else
				myButton:SetAlpha( XUB.GLOBALDIM_ALPHA );
			end
		end

		if myButton.is_STATE_BUFFED    then
			if ( XUB.HEALACTION == "Show as Disabled" ) then
				if myButton.isShowAlways then
					myButton:SetAlpha( XUB.COOLDOWN_ALPHA );
				else
					myButton:SetAlpha( XUB.INVALID_ALPHA );
					return;
				end
			else
				myButton:GetHighlightTexture():SetAlpha( 1 );       --makes the button react like it can be clicked ;-)
				if ( XUB.COLORUSAGE ~= "Transparency" ) then
					myButton:GetNormalTexture():SetVertexColor(XUB.COLORS["c_buffed"].r, XUB.COLORS["c_buffed"].g,XUB.COLORS["c_buffed"].b);
					myButton:SetAlpha( XUB.ENABLE_ALPHA * 0.7 );
				else
					myButton:SetAlpha( XUB.BUFFED_ALPHA );
				end
			end
		end

		if myButton.is_STATE_HEALTHOFF then
			if ( XUB.HEALACTION == "Show as Disabled" ) then
				if myButton.isShowAlways then
					myButton:SetAlpha( XUB.COOLDOWN_ALPHA );
				else
					myButton:SetAlpha( XUB.INVALID_ALPHA );
					return;
				end
			else  -- if XUB.HEALACTION == "Show as Buffed" then
				myButton:GetHighlightTexture():SetAlpha( 1 );       --makes the button react like it can be clicked ;-)
				myButton:SetAlpha( XUB.BUFFED_ALPHA );
			end
		end

		if myButton.is_STATE_COOLDOWN  then
			if ( myButton.State <  9 ) then        -- if the button would be usable if not on cooldown
				if ( XUB.COLORUSAGE ~= "Transparency" ) then
					myButton:GetNormalTexture():SetVertexColor(XUB.COLORS["c_cooldown"].r, XUB.COLORS["c_cooldown"].g,XUB.COLORS["c_cooldown"].b);
					myButton:SetAlpha( myButton:GetAlpha() * 0.8 );
				else
					myButton:SetAlpha( XUB.COOLDOWN_ALPHA );
				end
			end
		end
	end
end


function XUB_HandleBlinking(myButton)

	myButton:GetHighlightTexture():SetAlpha( 1 );       		--makes the button react like it can be clicked ;-)
	if ( XUB.time_for_blink >= 0.5 ) then
		myButton:GetHighlightTexture():SetAlpha( 1 );       --makes the button react like it can be clicked ;-)
		myButton:GetNormalTexture():SetVertexColor(1,1,1);
		myButton:SetAlpha( XUB.ENABLE_ALPHA );
	else
		if ( XUB.COLORUSAGE ~= "Transparency" ) then
			myButton:GetNormalTexture():SetVertexColor(XUB.COLORS["c_buffed"].r, XUB.COLORS["c_buffed"].g,XUB.COLORS["c_buffed"].b);
		end
		myButton:SetAlpha( 0.2 );
	end
end


function XUB_Set_CountdownText(myButton)

	if ( XUB.COUNTDOWN == 1 ) then
		if ( myButton.ShowCountdown ) then
			--if ( myButton.MyCooldown <= 5 ) then
			if ( myButton.MyCooldown <= 3 ) then
				myButton.CooldownFrame.text:SetTextColor(1, 0, 0, 1);   -- red
			else
				myButton.CooldownFrame.text:SetTextColor(1, 1, 1, 1);   -- white
			end
			myButton.CooldownFrame.text:SetText(XUB_Generate_CountdownText(myButton));
--		else
--			myButton.CooldownFrame.text:SetText("")
		end
	end
end

function XUB_Generate_CountdownText(myButton)

	local myReadableCooldown = "";

	if ( myButton.MyCooldown > 3599 ) then
		myReadableCooldown = "" ;
	elseif ( myButton.MyCooldown > 60 ) then
		myReadableCooldown = string.format("%d",myButton.MyCooldown/60) .. "m";
	elseif ( myButton.MyCooldown > 9.9 ) then
		myReadableCooldown = string.format("%d",math.ceil(myButton.MyCooldown));
	elseif ( myButton.MyCooldown > 0.01 ) then
		if XUB.ShowDecimals and ( myButton.MyCooldown <= 3 ) then
			myReadableCooldown = string.format("%.1f", myButton.MyCooldown)
		else
			myReadableCooldown = string.format("%d",math.ceil(myButton.MyCooldown));
		end
	else
		myReadableCooldown = "";
		myButton.CooldownFrame.text:SetTextColor(1, 1, 1, 1);
	end

	return myReadableCooldown;

end
