﻿local addonName, XUB = ...;

--[[
internals of all Buttons:

Buttons[1]: SpellID / ItemID (or "MacroID" for macros)
Buttons[2]: SpellTexture / ItemTexture / MacroIcon
Buttons[3]: name of the button
Buttons[4]: buttonType (spell, spellblink, buff, buffblink, debuff, debuffblink, health, healthblink, use or macro)
Buttons[5]: spellname / macroIconNumber / useItem
Buttons[6]: buff: condition / debuff: condition / health: threshold / macro: macrotext / useItem: condition

All non-custom buttons have buttonType 'spell'

Existing buttontypes are: 
Items: 'use'
Macros: 'macro'
Spells: 'spell', 'spellblink', 'buff', 'buffblink', 'debuff', 'debuffblink', 'health', 'healthblink'

--]]

local l_ID = 1;
local l_Texture = 2;
local l_ButtonName = 3;
local l_Type = 4;
local l_OldGroupName = "~first~group~";

local XUB_SS_SelectedSpell = 0;

local mySortedCustomButtons = {}


function XUB_SS_OnShow()

--  XUB_SelectSpell:SetAlpha(1);
	XUB_SS_SelectedSpell = 0;
	XUB_SelectSpellGroupName:SetText("");
	XUB_SelectSpellListLabel0Label:SetText("*** NO SELECTION ***");
	XUB_SelectSpellListTexture0Texture:SetTexture( "" );
	XUB_SetBackground(XUB_SelectSpell);

	XUB_SelectSpellTextPanelGroupEdit:SetText("");

	XUB_SelectSpell:Raise();

	XUB_GenerateDefaultSpellList()
	
	--- Generate sorted List of custom buttons
	local myDummyTable = {}
	for	i = 1,#XUB.CustomButtons do
		myDummyTable[XUB.CustomButtons[i][3]] = XUB.CustomButtons[i];	-- no .."~~"..i here, because the custom buttons must be unique anyway
	end
	mySortedCustomButtons = XUB_TableSort(myDummyTable);
	
	XUB_SelectSpellListSliderSlider:SetValue(1);
	
	local myMaxValue = #XUB.StandardButtons+#mySortedCustomButtons

	if myMaxValue < 10 then
	-- less then 10 buttons
		XUB_SelectSpellListSliderSlider:SetMinMaxValues(1,1);
		XUB_SelectSpellListSliderSlider:Disable();
	else
		XUB_SelectSpellListSliderSlider:SetMinMaxValues(1,myMaxValue-8);
		XUB_SelectSpellListSliderSlider:Enable();
	end

	XUB_SS_ShowSpells(1);
end

function XUB_SS_ShowSpells(aIndex)

	local myMaxValue = #XUB.StandardButtons+#mySortedCustomButtons;
	if myMaxValue > 9 then
		myMaxValue = 9;
	end
	
	for i=1,myMaxValue do
		XUB_SS_ShowASpell(i,aIndex+i-1);
	end
end

function XUB_SS_ShowASpell(aPosition,aEntryID)

	local myLabel = getglobal("XUB_SelectSpellListLabel" .. aPosition .. "Label" );
	local myTexture = getglobal("XUB_SelectSpellListTexture" .. aPosition .. "Texture" );

	local mySelectedLabel = getglobal("XUB_SelectSpellListLabel0Label" );
	local mySelectedTexture = getglobal("XUB_SelectSpellListTexture0Texture" );
	
	myTexture:SetTexture( XUB_SS_GetTexture(aEntryID) );
	myLabel:SetText( XUB_SS_GetEntryText(aEntryID) );
	
	if ( aEntryID == XUB_SS_SelectedSpell ) then
		mySelectedTexture:SetTexture( XUB_SS_GetTexture(aEntryID) );
		mySelectedLabel:SetText( XUB_SS_GetEntryText(aEntryID) );

		myTexture:SetVertexColor(0.3,0.3,0.3);
	else
		myTexture:SetVertexColor(1,1,1);
	end
	
end

function XUB_SS_GetTexture(aEntryID)
	local myEntry;
	
	if ( aEntryID <= #XUB.StandardButtons ) then
		myEntry = XUB.StandardButtons[aEntryID];
	else
		myEntry = mySortedCustomButtons[aEntryID-#XUB.StandardButtons];
	end
	
	
	if ( myEntry[l_Type] == "macro" ) then
		-- Required call		
		GetNumMacroIcons();
		-- aThreshold holds the id for the macro.
		return GetMacroIconInfo( myEntry[5] );
	elseif ( myEntry[l_Type] == "use" ) then
		local USEitemName, USEitemLink, USEitemRarity, USEitemLevel, USEitemMinLevel, USEitemType, USEitemSubType, USEitemStackCount, USEitemEquipLoc, USEitemTexture = GetItemInfo(myEntry[5]) 

		if ( USEitemTexture == nil ) then
			USEitemTexture = "Interface\\Icons\\INV_Misc_QuestionMark";
		elseif ( USEitemTexture == "" ) then 
			USEitemTexture = "Interface\\Icons\\INV_Misc_QuestionMark";
		end
	
		return USEitemTexture;
	else -- must be a spell of some kind
		local _, _, mySpellIcon = GetSpellInfo( myEntry[5] )
		return mySpellIcon
	end
end

function XUB_SS_GetEntryText(aEntryID)

	local myEntry;
	
	if ( aEntryID <= #XUB.StandardButtons ) then
		myEntry = XUB.StandardButtons[aEntryID];
	else
		myEntry = mySortedCustomButtons[aEntryID-#XUB.StandardButtons];
	end
	
	if ( myEntry[l_Type] == "macro" ) then
		return myEntry[6];
	elseif ( myEntry[l_Type] == "use" ) then
		return "Use item '" .. myEntry[5] .. "'";
	else
		local myBaseText = myEntry[5];
		
		if ( myEntry[l_Type] == "spell" ) then
			return myBaseText;
		elseif ( myEntry[l_Type] == "always" ) then
			return myBaseText .. "[Show always]";
		elseif ( myEntry[l_Type] == "spellblink" ) then
			return myBaseText .. "[Start blinking if not buffed with spell]";
		elseif ( myEntry[l_Type] == "health" ) or ( myEntry[2] == "heal" ) then
			local myThreshold = tonumber(myEntry[6]);
			if ( myThreshold > 0 ) then
				return myBaseText .. " [Show if damage > " .. myThreshold .. "]";
			else
				return myBaseText .. " [Show if health < " .. -myThreshold .. "]";
			end
		end
		local myCondition = myEntry[6];
		if myCondition:sub(1,1) == "$" then
			mycondition = "own: " .. myCondition:sub(2);
		elseif myCondition:sub(1,1) == "!" then
			mycondition = ": " .. myCondition:sub(2);
		else
			mycondition = ": " .. myCondition;
		end
		if ( myEntry[l_Type]:sub(1,4) == "buff" ) then
			myBaseText = myBaseText .. " [Hide if buffed with " .. mycondition;
		elseif ( myEntry[l_Type]:sub(1,6) == "debuff" ) then
			myBaseText = myBaseText .. " [Show if debuffed with " .. mycondition;
		end
		if string.find(myEntry[l_Type],"blink") then
			return myBaseText .. ". If shown, start blinking.]";
		else
			return myBaseText .. ".]";
		end
	end
end

function XUB_SS_AppendGroup(aEntryID)
		
	if ( aEntryID <= #XUB.StandardButtons ) then
		myEntry = XUB.StandardButtons[aEntryID];
	else
		myEntry = mySortedCustomButtons[aEntryID-#XUB.StandardButtons];
	end
	
	local myGroupName = XUB_SelectSpellGroupName:GetText();
	local myEditboxText = XUB_SelectSpellTextPanelGroupEdit:GetText();
	
	if ( myGroupName == "" ) then
		myGroupName = "UnknownGroup";
	end
	
	if ( myEditboxText ) and ( myEditboxText ~= "" ) then
		XUB_SelectSpellTextPanelGroupEdit:SetText( myEditboxText .. "\n" .. myGroupName .. "," .. myEntry[l_ButtonName] .. "," .. strtrim(XUB_SelectSpellTargetText:GetText()) );
	else
		XUB_SelectSpellTextPanelGroupEdit:SetText( myGroupName .. "," .. myEntry[l_ButtonName] .. "," .. strtrim(XUB_SelectSpellTargetText:GetText()) );
	end

	-- if we have already some text in the box  
	--     if we have a new groupname then   	add two CRs
	--     else     add one single CR
	
	
	local myNewText = myGroupName;
	if ( myEditboxText ) and ( myEditboxText ~= "" ) then
		if myGroupName ~= l_OldGroupName then
				myNewText = "\n\n" .. myGroupName;
		else
				myNewText = "\n" .. myGroupName;
		end
	end
	XUB_SelectSpellTextPanelGroupEdit:SetText( myEditboxText .. myNewText .. "," .. myEntry[l_ButtonName] .. "," .. strtrim(XUB_SelectSpellTargetText:GetText()) );

	l_OldGroupName = myGroupName;
	
end

function XUB_SS_OnValueChanged()
	local myItemNbr = XUB_SelectSpellListSliderSlider:GetValue();
	XUB_SS_ShowSpells(myItemNbr);
end

function XUB_SS_SetSpell(aPosition)
	XUB_SS_SelectedSpell = XUB_SelectSpellListSliderSlider:GetValue() + aPosition - 1;
	XUB_SS_OnValueChanged();
		
	XUB_SS_AppendGroup(XUB_SS_SelectedSpell);
end

function XUB_SelectSpell_OnClickOK()
	XUB_SelectSpell:Hide();
	XUB_Config_AddNewContentToGroups(XUB_SelectSpellTextPanelGroupEdit:GetText());
	local SCR = getglobal(XUB_SelectSpellReturnScreenLabel:GetText());
	SCR:Show();
end

function XUB_SelectSpell_OnClickCancel()
	XUB_SelectSpell:Hide();
	local SCR = getglobal(XUB_SelectSpellReturnScreenLabel:GetText());
	SCR:Show();
end
